/*
 * $Id: ConvertToOclStringAction.java,v 1.2 2004/11/03 14:59:16 haensgen Exp $
 */

package vocleditor.actions;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;

import VOCLModel.VOCLDiagram;
import VoclToOclConverter.InvalidModelException;
import VoclToOclConverter.ModelConverter;
import VoclToOclConverter.ModelConverterException;

/**
 * Action which triggers the VisualOCL EMF model to OCL string converter. The
 * ActionBarContributer which registers this action must provide a reference to
 * the VOCLDiagram instance of the currently active EditorPart. This can be set 
 * via setModelRoot().
 *
 * @see OclConstraintModel.converter.ModelConverter
 */
public class ConvertToOclStringAction extends Action {

	// The VOCLDiagram instance of the currently active EditorPart
	private VOCLDiagram modelRoot;

	/**
	 * Creates the action.
	 */
	public ConvertToOclStringAction() {
		modelRoot = null;
	}

	/**
	 * Sets the root of the EMF model instance of the currently active 
	 * EditorPart.
	 *
	 * @param modelRoot
	 *     The root.
	 */
	public void setModelRoot(VOCLDiagram modelRoot) {
		this.modelRoot = modelRoot;
	}

	/**
	 * Triggers the VisualOCL EMF model to OCL string converter. The OCL
	 * expression which belongs to the NavExp instance of the currently active
	 * EditorPart will be converted to an OCL string and displayed in a message
	 * box, and additionally printed to stdout.
	 */
	public void run() {
		if (modelRoot == null)
			return;
		
		String[] oclStrings = null;
		try {
			ModelConverter converter = new ModelConverter(modelRoot);
			oclStrings = converter.convertToOclStrings();
		} catch (InvalidModelException ex) {
			IStatus status = ex.getStatus();
			MultiStatus dlgStatus = new MultiStatus(
				status.getPlugin(),
				status.getCode(),
				status.getChildren(),
				"See \"Details\" for a list of error messages occured " +
				"during conversion.",
				null);
			ErrorDialog.openError(
				null,
				"VOCL to OCL String Converter: Error",
				"There is an error in the VOCLModel.",
				dlgStatus);
			return;
		} catch (ModelConverterException ex) {
			IStatus status = ex.getStatus();
			MessageDialog.openError(
				null,
				"VOCL to OCL String Converter",
				ex.toString());
			return;
		}
			
		if (oclStrings.length == 0) {
			System.out.println("VOCL to OCL String Converter: " +
				"The expression is empty.");
			MessageDialog.openInformation(null, 
				"VOCL to OCL String Converter", 
				"The expression is empty.");
		} else {
			String oclString = "";
			for (int i = 0; i < oclStrings.length; i++) {
				if (i > 0)
					oclString += "\n\n";
				oclString += oclStrings[i];
			}
			if (oclStrings.length == 1)
				System.out.println("OCL String:\n\n" + oclString);
			else
				System.out.println("OCL Strings:\n\n" + oclString);
			MessageDialog.openInformation(null, "OCL String: ", oclString);
		}
	}
}
