package vocleditor.actions;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import vocleditor.command.CreateClassifierRoleSetCommand;
import vocleditor.command.CreateClassifierRoleSimpleCommand;
import vocleditor.command.CreateConditionCommand;
import vocleditor.command.EditClassifierRoleSimpleCommand;
import vocleditor.edit.parts.ClassifierRoleSetEditPart;
import vocleditor.edit.parts.ClassifierRoleSimpleEditPart;
import vocleditor.edit.parts.ConditionEditPart;
import vocleditor.edit.parts.NavExpEditPart;
import vocleditor.model.ModelCreationFactory;
import VOCLModel.ClassifierRole;
import VOCLModel.ClassifierRoleSet;
import VOCLModel.Condition;
import VOCLModel.NavExp;

/**
 * 
 * Class       : EditClassifierNode<br/>
 * Package     : vocleditor.actions<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : An actionclass for editing ClassifierNodes.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class CreateConditionAction extends SelectionAction {

	public static final String CONDITION = "ConditionID";

	/**
	 * Constructor.
	 * @param part the current Editorpart
	 */
	public CreateConditionAction(IWorkbenchPart part) {
		super(part);

		setText("Create a Condition");
		setId(CONDITION);
	}

	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return canPerformAction();
	}

	/**
	 * Set the current active part. In this case the VOCLEditor.
	 * @param part the current editor
	 */
	public void setActivePart(IEditorPart part) {
		setActivePart(part);
	}

	/**
	 * Returns whether the Action can be performed. It can be performed 
	 * if a ClassifierNode is selected.
	 * @return
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
//			System.out.println(part.toString());
			if ((part instanceof NavExpEditPart))
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * Returns the ClassifierRole oder ClassifierRoleSetCommand. It depends
	 * on which EditpPart was selected.
	 * @return the command
	 */
	private Command getCommand() {
		Object selection =
			((StructuredSelection) getSelection()).getFirstElement();
		NavExpEditPart part = (NavExpEditPart)selection;
		ModelCreationFactory mfc = new ModelCreationFactory(Condition.class);
		Condition condition = (Condition) mfc.getNewObject();
		CreateConditionCommand command = new CreateConditionCommand(part.getShell());
		command.setParent(part.getModel());
		command.setCondition(condition);
		return command;
	}

	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		execute(getCommand());
	}

}
