package vocleditor.actions;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import vocleditor.command.CreateClassifierRoleSetCommand;
import vocleditor.command.CreateClassifierRoleSimpleCommand;
import vocleditor.command.EditClassifierRoleSimpleCommand;
import vocleditor.edit.parts.ClassifierRoleSetEditPart;
import vocleditor.edit.parts.ClassifierRoleSimpleEditPart;
import VOCLModel.ClassifierRole;
import VOCLModel.ClassifierRoleSet;

/**
 * 
 * Class       : EditClassifierNode<br/>
 * Package     : vocleditor.actions<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : An actionclass for editing ClassifierNodes.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class EditClassifierNode extends SelectionAction {

	public static final String CLASSIFIER = "ClassifierNode";
	private ClassifierRoleSimpleEditPart classifierEditPart;
	private ClassifierRoleSetEditPart classifierSetEditPart;

	/**
	 * Constructor.
	 * @param part the current Editorpart
	 */
	public EditClassifierNode(IWorkbenchPart part) {
		super(part);

		setText("Edit ClassifierNode");
		setId(CLASSIFIER);
	}

	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return canPerformAction();
	}

	/**
	 * Set the current active part. In this case the VOCLEditor.
	 * @param part the current editor
	 */
	public void setActivePart(IEditorPart part) {
		setActivePart(part);
	}

	/**
	 * Returns whether the Action can be performed. It can be performed 
	 * if a ClassifierNode is selected.
	 * @return
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if ((part.getModel() instanceof ClassifierRole))
				return true;
			else if ((part.getModel() instanceof ClassifierRoleSet))
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * Returns the ClassifierRole oder ClassifierRoleSetCommand. It depends
	 * on which EditpPart was selected.
	 * @return the command
	 */
	private Command getCommand() {
		Object selection =
			((StructuredSelection) getSelection()).getFirstElement();
		if (selection instanceof ClassifierRoleSetEditPart) {
			classifierSetEditPart = (ClassifierRoleSetEditPart) selection;
			Shell shell = classifierSetEditPart.getShell();
			CreateClassifierRoleSetCommand create =
				new CreateClassifierRoleSetCommand(shell);
			create.setClassifierRoleSet(
				classifierSetEditPart.getClassifierRoleSet());
			create.setEdit(true);
			return create;
		} else {
			classifierEditPart = (ClassifierRoleSimpleEditPart) selection;
			Shell shell = classifierEditPart.getShell();
			EditClassifierRoleSimpleCommand create =
				new EditClassifierRoleSimpleCommand(shell);
			create.setClassifierRole(
				classifierEditPart.getClassifierRole());
			return create;
		}
	}

	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		execute(getCommand());
	}

}
