package vocleditor.actions;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import vocleditor.command.CreateClassifierRoleSetCommand;
import vocleditor.command.CreateClassifierRoleSimpleCommand;
import vocleditor.command.EditClassifierRoleSimpleCommand;
import vocleditor.command.EditConstraintCommand;
import vocleditor.edit.parts.ClassifierRoleSetEditPart;
import vocleditor.edit.parts.ClassifierRoleSimpleEditPart;
import vocleditor.edit.parts.ConstraintEditPart;
import vocleditor.edit.parts.VOCLDiagrammEditPart;
import VOCLModel.ClassifierRole;
import VOCLModel.ClassifierRoleSet;
import VOCLModel.Constraint;
import VOCLModel.VOCLDiagram;

/**
 * 
 * Class       : EditClassifierNode<br/>
 * Package     : vocleditor.actions<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : An actionclass for editing ClassifierNodes.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class EditConstraintAction extends SelectionAction {

	public static final String CLASSIFIER = "Constraint";
	private ConstraintEditPart constraint;
	/**
	 * Constructor.
	 * @param part the current Editorpart
	 */
	public EditConstraintAction(IWorkbenchPart part) {
		super(part);

		setText("Edit Constraint");
		setId(CLASSIFIER);
	}

	/**
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		return canPerformAction();
	}

	/**
	 * Set the current active part. In this case the VOCLEditor.
	 * @param part the current editor
	 */
	public void setActivePart(IEditorPart part) {
		setActivePart(part);
	}

	/**
	 * Returns whether the Action can be performed. It can be performed 
	 * if a ClassifierNode is selected.
	 * @return
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
		List parts = getSelectedObjects();
		for (int i = 0; i < parts.size(); i++) {
			Object o = parts.get(i);
			if (!(o instanceof EditPart))
				return false;
			EditPart part = (EditPart) o;
			if ((part.getModel() instanceof Constraint))
				return true;
			else
				return false;
		}
		return true;
	}

	/**
	 * Returns the ClassifierRole oder ClassifierRoleSetCommand. It depends
	 * on which EditpPart was selected.
	 * @return the command
	 */
	private Command getCommand() {
		Object selection =
			((StructuredSelection) getSelection()).getFirstElement();
		if(selection instanceof ConstraintEditPart){
			constraint = (ConstraintEditPart) selection;
			Shell shell = constraint.getShell();
			EditConstraintCommand c = new EditConstraintCommand(shell);
			c.setConstraint((Constraint) constraint.getModel());
			c.setParent((VOCLDiagram) ((VOCLDiagrammEditPart)constraint.getParent()).getModel());
			return c;
		}
		return null;
//		if (selection instanceof ClassifierRoleSetEditPart) {
//			classifierSetEditPart = (ClassifierRoleSetEditPart) selection;
//			Shell shell = classifierSetEditPart.getShell();
//			CreateClassifierRoleSetCommand create =
//				new CreateClassifierRoleSetCommand(shell);
//			create.setClassifierRoleSet(
//				classifierSetEditPart.getClassifierRoleSet());
//			create.setEdit(true);
//			return create;
//		}
	}

	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run() {
		execute(getCommand());
	}

}
