package vocleditor.actions;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.ui.actions.ActionFactory;

/** 
 * Class       : VOCLContextMenuProvider<br/>
 * Package     : vocleditor.actions<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This is the context menu provider for our editor.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class VOCLContextMenuProvider extends ContextMenuProvider {
	/** the action registry */
	private final ActionRegistry actionRegistry;

	/**
	 * Creates a new WorkflowEditorContextMenuProvider instance.
	 * @param viewer
	 */
	public VOCLContextMenuProvider(
		EditPartViewer viewer,
		ActionRegistry actionRegistry) {
		super(viewer);
		this.actionRegistry = actionRegistry;
		
	}

	/**
	 * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void buildContextMenu(IMenuManager menuManager) {
		GEFActionConstants.addStandardActionGroups(menuManager);
		appendActionToAddGroup(menuManager,CreateConditionAction.CONDITION);
		appendActionToAddGroup(menuManager,EditConstraintAction.CLASSIFIER);
		appendActionToUndoGroup(menuManager, ActionFactory.UNDO.getId());
		appendActionToUndoGroup(menuManager, ActionFactory.REDO.getId());

		appendActionToEditGroup(menuManager, ActionFactory.COPY.getId());
		appendActionToEditGroup(menuManager, ActionFactory.PASTE.getId());
		appendActionToEditGroup(menuManager, ActionFactory.DELETE.getId());
		appendActionToEditGroup(menuManager, GEFActionConstants.DIRECT_EDIT);
		appendActionToAddGroup(menuManager, EditClassifierNode.CLASSIFIER);
		appendActionToAddGroup(menuManager,NotVOCLExpFigureAction.ID);
		
		appendActionToMenu(
			menuManager,
			ActionFactory.SAVE.getId(),
			GEFActionConstants.GROUP_SAVE);
	}

	/**
	 * Appends the alignment subment.
	 * @param menuManager
	 */
	private void appendAlignmentSubmenu(IMenuManager menuManager) {
		// Alignment Actions
		MenuManager submenu = new MenuManager("Align");

		IAction action =
			getActionRegistry().getAction(GEFActionConstants.ALIGN_LEFT);
		if (null != action && action.isEnabled())
			submenu.add(action);

		action = getActionRegistry().getAction(GEFActionConstants.ALIGN_CENTER);
		if (null != action && action.isEnabled())
			submenu.add(action);

		action = getActionRegistry().getAction(GEFActionConstants.ALIGN_RIGHT);
		if (null != action && action.isEnabled())
			submenu.add(action);

		action = getActionRegistry().getAction(GEFActionConstants.ALIGN_TOP);
		if (null != action && action.isEnabled())
			submenu.add(action);

		action = getActionRegistry().getAction(GEFActionConstants.ALIGN_MIDDLE);
		if (null != action && action.isEnabled())
			submenu.add(action);

		action = getActionRegistry().getAction(GEFActionConstants.ALIGN_BOTTOM);
		if (null != action && action.isEnabled())
			submenu.add(action);

		if (!submenu.isEmpty())
			menuManager.appendToGroup(GEFActionConstants.GROUP_REST, submenu);
	}

	/**
	 * Returns the action registry.
	 * @return the action registry
	 */
	protected ActionRegistry getActionRegistry() {
		return actionRegistry;
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToMenu(
		IMenuManager menu,
		String actionId,
		String menuGroup) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action && action.isEnabled())
			menu.appendToGroup(menuGroup, action);
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToUndoGroup(IMenuManager menu, String actionId) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action && action.isEnabled())
			menu.appendToGroup(GEFActionConstants.GROUP_UNDO, action);
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToEditGroup(IMenuManager menu, String actionId) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action && action.isEnabled())
			menu.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToAddGroup(IMenuManager menu, String actionId) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action && action.isEnabled())
			menu.appendToGroup(GEFActionConstants.GROUP_ADD, action);
	}

}
