package vocleditor.command;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;

import VOCLModel.AssociationRole;
/**
 * 
 * Class       : BendpointCommand<br/>
 * Package     : vocleditor.command<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Superclass for creation, deletion and moving Bendpoints.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class BendpointCommand extends Command {

	//index of the Bendpoint (each Bendpoint has one)
	protected int index;
	//the location of the Bendpoint
	protected Point location;
	//the associationrole to that the Bendpoint belongs to
	protected AssociationRole associationRole;
	//help constants
	private Dimension d1, d2;

	/**
	 * Returns the relative Dimension to the source ClassifierRoleNode.
	 * @return the dimension.
	 */
	protected Dimension getFirstRelativeDimension() {
		return d1;
	}

	/**
	 * Returns the relative Dimension to the target ClassifierRoleNode.
	 * @return the dimension
	 */
	protected Dimension getSecondRelativeDimension() {
		return d2;
	}

	/**
	 * The index of the Bendpoint.
	 * @return the index
	 */
	protected int getIndex() {
		return index;
	}

	/**
	 * Return the location of the Bendpoint.
	 * @return the location
	 */
	protected Point getLocation() {
		return location;
	}

	/**
	 * Returns the Associationrole to which the Bendpoint belongs to.
	 * @return the Associationrole
	 */
	protected AssociationRole getAssociationRole() {
		return associationRole;
	}

	/*
	 * Redo the Command 
	 */
	public void redo() {
		execute();
	}

	/**
	 * Sets the relative dimesions of the Bendpoint towards the classifiernodes
	 * @param dim1 Relative Dimension to the source classifiernode
	 * @param dim2 Relative Dimension to the target classifiernode
	 */
	public void setRelativeDimensions(Dimension dim1, Dimension dim2) {
		d1 = dim1;
		d2 = dim2;
	}

	/**
	 * Sets the BendpointIndex. Each Bendpoint has exactly one Index
	 * @param i the index
	 */
	public void setIndex(int i) {
		index = i;
	}

	/**
	 * Sets the Location of a Bendpoint.
	 * @param p the location
	 */
	public void setLocation(Point p) {
		location = p;
	}

	/**
	 * Sets the current associationrole the Bendpoint belongs to.
	 * @param associationrole the current associationrole
	 */
	public void setAssociationRole(AssociationRole associationrole) {
		associationRole = associationrole;
	}
}
