package vocleditor.command;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.Command;

import vocleditor.figures.ConstraintFigure;
import VOCLModel.Constraint;

/**
 * 
 * Class       : ConstraintMoveCommand<br/>
 * Package     : vocleditor.command<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This class has functionality for moving and resizing a Constraint.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class ConstraintMoveCommand extends Command
{
	private static final String locationLabel = "Change Constraint Position";
	private static final String resizeLabel = "Constraint Resize Command";
	//the old position
	private Point newPos;
	//the new position
	private Point oldPos;
	private Constraint constraint;
	private ConstraintFigure figure;
	private Object cons;
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() 
	{
		//resize the Constraintfigure
		if(((Rectangle)cons).getSize().width != -1){
			figure.setPreferredSize(((Rectangle)cons).getSize());
		}
		oldPos = new Point(constraint.getX(), constraint.getY());
		constraint.setX(newPos.x);
		constraint.setY(newPos.y);
		constraint.setHeight(((Rectangle)cons).getSize().height);
		constraint.setWidth(((Rectangle)cons).getSize().width);
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#getLabel()
	 */
	public String getLabel()
	{
		return resizeLabel;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() 
	{
		constraint.setX(newPos.x);
		constraint.setY(newPos.y);
	}
	
	/**
	 * @param r
	 */
	/**
	 * the current constraint for moving and resizing the Constraintfigure
	 * @param r
	 */
	public void setLocation(Rectangle r)
	{
		setLocation(r.getLocation());
		cons = r;
	}
	
	/**
	 *  set the current location of the Constraintfigure
	 * @param p
	 */
	public void setLocation(Point p) 
	{
		newPos = p;
	}
	
	/**
	 * Sets the current Constraint from the modell.
	 * @param constraint the Constraint
	 */
	public void setPart(Constraint constraint) 
	{
		this.constraint = constraint;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() 
	{
		constraint.setX(oldPos.x);
		constraint.setY(oldPos.y);
	}
	/**
	 * Sets the current Constraintfigure for resizing.
	 * @param figure
	 */
	public void setFigure(ConstraintFigure figure) {
		this.figure = figure;
	}

}
