package vocleditor.command;

import org.eclipse.gef.commands.Command;
import org.eclipse.swt.widgets.Shell;

import vocleditor.dialogs.CreateBoolExpressionDialog;
import vocleditor.model.Helpers;
import VOCLModel.BoolExp;
import VOCLModel.BoolExpKind;

/**
 * 
 * Class       : CreateBendpointCommand<br/>
 * Package     : vocleditor.command<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 28.06.2004<br/>
 * Description : Command for Creating a new BoolExpression..<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class CreateBoolExpCommand extends Command {

	//the new created BoolExp
	BoolExp boolexp;
	//the indexed-position in the parent figure
	int position;
	//the parent of the BoolExp
	Object parent;
	//needed for displaying
	Shell shell;

	/**
	 * Constructor
	 * @param shell
	 */
	public CreateBoolExpCommand(Shell shell) {
		this.shell = shell;
	}

	/**
	 * the Position in the parent Figure
	 * @param p
	 */
	public void setPosition(int p) {
		position = p;
	}
	/**
	 * The BoolExpression that is created.
	 * @param exp
	 */
	public void setBoolExp(BoolExp exp) {
		boolexp = exp;
	}

	/**
	 * Set the parent of this BoolExpression.
	 * @param parent
	 */
	public void setParent(Object parent) {
		this.parent = parent;
	}

	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		if(Helpers.constraintHasAlreadyVOCLExpression(shell,parent) == 1){
			return;
		}
		CreateBoolExpressionDialog d = new CreateBoolExpressionDialog(shell, 0);
		d.open();
		if (d.isCancelled())
			return;
		//set the current operation
		switch (d.getOperation()) {
			case 0 :
				boolexp.setKind(BoolExpKind.OR_LITERAL);
				break;
			case 1 :
				boolexp.setKind(BoolExpKind.XOR_LITERAL);
				break;
			case 2 :
				boolexp.setKind(BoolExpKind.IMPLIES_LITERAL);
				break;
		}

		boolexp.setIsVertical(d.isSelected());
		//the part of the parent figure in which this figure is placed
		boolexp.setIndex(position);
		//insert the BoolExpression into the model 
		Helpers.addToParent(boolexp, parent, position);

	}

	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		//remove the BoolExp from the modell
		Helpers.removeFromParent(parent, position);
	}

	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		//insert the boolExp into the modell
		Helpers.addToParent(boolexp, parent, position);
	}

}
