package vocleditor.command;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.eclipse.swt.widgets.Shell;

import vocleditor.dialogs.CreateNewClassifierRoleSetDialog;
import vocleditor.model.Helpers;
import vocleditor.model.NavExpConstants;
import vocleditor.model.VOCLModelManager;
import VOCLModel.Attribute;
import VOCLModel.ClassifierRoleSet;
import VOCLModel.NavExp;
import VOCLModel.OpKindEnum;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModelFactory;
import VOCLModel.VariableDeclaration;

/**
 * 
 * Class       : CreateClassifierRoleSetCommand<br/>
 * Package     : vocleditor.command<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Command for Creating a new ClassifierRoleSet.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class CreateClassifierRoleSetCommand extends AbstractCreateClassifierRoleCommand {
	// Label to show in editor palette
	private static final String label = "Set Classifier Role";

	// Description shown in the editor
	private static final String description =
		"Creates a new set-classifierrole";
	// Data for execute()
	private ClassifierRoleSet classifierRole = null;
	private VariableDeclaration variableDeclaration = null;
	private VariableDeclaration operationVarDeclaration = null;
	private VariableDeclaration iteratorVarDeclaration = null;
	//indicates whether the classifierRole is edited / created
	private boolean isEdit;

	//list to store the attributes
	private List attrList = new Vector();

	/**
	 * Creates the command.
	 */
	public CreateClassifierRoleSetCommand() {
		setLabel(label);
	}

	/**
	 * Creates the command and uses the given parent shell to display the 
	 * dialog box.
	 * 
	 * @param shell
	 *     The shell.
	 */
	public CreateClassifierRoleSetCommand(Shell shell) {
		setLabel(label);
		this.shell = shell;
	}

	/**
	 * Sets the current ClassifierRoleSet.
	 * @param classifierRole
	 */
	public void setClassifierRoleSet(ClassifierRoleSet classifierRole) {
		this.classifierRole = classifierRole;
	}

	/**
	 * Return the current parent (Navexpression)
	 * @return the Navexpression.
	 */
	public NavExp getParent() {
		return parent;
	}

	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		if (Helpers.constraintHasAlreadyVOCLExpression(shell, navexpParent)
			== 1) {
			return;
		}

		VOCLModelFactory mfc = VOCLModelManager.getFactory();

		//open a dialog for editing the settings
		CreateNewClassifierRoleSetDialog d =
			new CreateNewClassifierRoleSetDialog(shell);
		if (parent == null && classifierRole.eContainer() == null) {
			parent = mfc.createNavExp();
			parent.setIndex(position);
			Helpers.addToParent(parent, navexpParent, position);
		} else if (parent == null && classifierRole.eContainer() != null) {
			parent = (NavExp) classifierRole.eContainer();
		}
		//set the parent into the dialog
		d.setNavexp(parent);

		//if it is in edit - modus
		if (isEdit) {
			//set current classifierRole to the dialog
			d.setClassifierRole(classifierRole);
			//remove all from the model
			removeFromModel(classifierRole);
		}

		d.open();
		if (isEdit) {
			classifierRole.setClassifierVar(null);
			classifierRole.getAttribute().clear();
			classifierRole.setOpKindEnum(null);
		}
		if (d.isCanceled()) {
			Helpers.removeFromParent(navexpParent, position);
			return;
		}
		//get the data of the dialog
		//type of the role
		String varType = d.getRoleType();
		if (varType == null)
			varType = "";
		else
			varType = varType.trim();

		//name of the role
		String varName = d.getRoleName();
		if (varName == null)
			varName = "";
		else
			varName = varName.trim();

		//the variable of the call
		String resultName = d.getCallVarName();
		resultName = resultName.trim();

		//the variable of the iteration process in the sum-function
		String iteratorName = d.getSumVarName();

		//
		String resultType = d.getResultType();
		resultType = resultType.trim();

		// Set up classifier role
		classifierRole.setType(varType);
		classifierRole.setName(varName);
		

		//get the attributes of the ClassifierRoleSet
		attributes = d.getAttributes();
		// Create attributes
		if (attributes.size() > 0) {
			attrList = new Vector();
			for (Iterator it = attributes.iterator(); it.hasNext();) {
				Attribute attr = mfc.createAttribute();
				attr.setName((String) it.next());

				String attrVarType = (String) it.next();
				attr.setType(attrVarType);
				attr.setValue((String) it.next());
				String attrVarName = (String) it.next();

				//if it has Variable in an Attribute
				if (attrVarName != "") {
					VariableDeclaration vardeclar =
						mfc.createVariableDeclaration();
					vardeclar.setType(attrVarType);
					vardeclar.setVarName(attrVarName);
					parent.getVariableDeclaration().add(vardeclar);
					attr.setVariableDeclaration(vardeclar);
				}
				//insert attribute into modell
				classifierRole.getAttribute().add(attr);
				attrList.add(attr);
			}
		}
		if (resultType.equals("EMPTY") || resultType.equals("NOT_EMPTY"))
			classifierRole.setOperationKind(resultType);

		//if operation is size or sum
		if (resultType.equals(NavExpConstants.SIZE)
			|| resultType.equals(NavExpConstants.SUM)) {
			//create a new VariableDeclaration for the call variable
			operationVarDeclaration = mfc.createVariableDeclaration();
			operationVarDeclaration.setVarName(resultName);
			operationVarDeclaration.setType(
				NavExpConstants.TYPE_INTEGER_AS_STRING);
			OpKindEnum opkindenum = mfc.createOpKindEnum();
			opkindenum.setCallVar(operationVarDeclaration);

			if (resultType.equals(NavExpConstants.SUM)) {
				//create a new VariableDeclaration for the iterator variable
				String iter = d.getSumVarName();
				VariableDeclaration dec = null;
				for(Iterator it = classifierRole.getAttribute().iterator(); it.hasNext();){
					Attribute attr = (Attribute) it.next();
					if(attr.getVariableDeclaration() != null){
						if(attr.getVariableDeclaration().getVarName().equals(iter)){
							dec = attr.getVariableDeclaration();
						}
					}
				}
//				iteratorVarDeclaration = mfc.createVariableDeclaration();
//				iteratorVarDeclaration.setVarName(iteratorName);
//				iteratorVarDeclaration.setType(
//					NavExpConstants.TYPE_INTEGER_AS_STRING);
				opkindenum.setIteratorVar(dec);
			}
			//set up the operation kind 
			classifierRole.setOpKindEnum(opkindenum);
			classifierRole.setOperationKind(resultType);
		}

		//Creates new VariableDeclaration if there exists a name and a type
		//for the ClassifierRoleSet
		if (!varName.equals("")) {
			variableDeclaration = mfc.createVariableDeclaration();
			variableDeclaration.setVarName(varName);
			variableDeclaration.setType(varType);
			classifierRole.setClassifierVar(variableDeclaration);
		}

		insertIntoModel();
	}

	/**
	 * Insert all Elements into the model
	 */
	private void insertIntoModel() {
		if (!isEdit)
			parent.getClassifierNode().add(classifierRole);

		if (variableDeclaration != null)
			parent.getVariableDeclaration().add(variableDeclaration);
		if (operationVarDeclaration != null) {
			parent.getVariableDeclaration().add(operationVarDeclaration);
		}
//		if (iteratorVarDeclaration != null) {
//			parent.getVariableDeclaration().add(iteratorVarDeclaration);
//		}
		//		set the location for the classifierrole in the NavExp
		if (loc != null) {
			classifierRole.setX(loc.x);
			classifierRole.setY(loc.y);
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		if (parent.getClassifierNode().size() == 0) {
			Helpers.addToParent(
				parent,
				(VOCLExpression) navexpParent,
				parent.getIndex());
		}
		insertIntoModel();
		if (attrList.size() > 0) {
			for (Iterator it = attrList.iterator(); it.hasNext();) {
				Attribute attribute = (Attribute) it.next();
				if (attribute.getVariableDeclaration() != null) {
					parent.getVariableDeclaration().add(
						attribute.getVariableDeclaration());
				}
			}
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		parent.getClassifierNode().remove(classifierRole);
		if (variableDeclaration != null)
			parent.getVariableDeclaration().remove(variableDeclaration);

		if (operationVarDeclaration != null) {
			parent.getVariableDeclaration().remove(operationVarDeclaration);
		}

		if (iteratorVarDeclaration != null) {
			parent.getVariableDeclaration().remove(iteratorVarDeclaration);
		}

		//remove all variabledeclarations of the attributes
		if (attrList.size() > 0) {
			for (Iterator it = attrList.iterator(); it.hasNext();) {
				Attribute attribute = (Attribute) it.next();
				if (attribute.getVariableDeclaration() != null) {
					parent.getVariableDeclaration().remove(
						attribute.getVariableDeclaration());
				}
			}
		}

		//delete the Navexpression if there does not exist any ClassifierNodes anymore
		if (parent.getClassifierNode().size() == 0) {
			Object navexpParent = parent.eContainer();
			Helpers.removeFromParent(navexpParent, parent.getIndex());
		}
	}
	/**
	 * @param b
	 */
	public void setEdit(boolean b) {
		isEdit = b;
	}

	/**
	 * Help method which deletes all Elements that belongs to
	 * the ClassifierRoleSet if it is in edit modus.
	 * @param classifier
	 */
	private void removeFromModel(ClassifierRoleSet classifier) {
		if (classifier.getClassifierVar() != null) {
			parent.getVariableDeclaration().remove(
				classifier.getClassifierVar());
		}

		if (classifier.getAttribute().size() > 0) {
			List list = new Vector();
			for (Iterator it = classifier.getAttribute().iterator();
				it.hasNext();
				) {
				Attribute attr = (Attribute) it.next();
				list.add(attr);
				if (attr.getVariableDeclaration() != null) {
					parent.getVariableDeclaration().remove(
						attr.getVariableDeclaration());
				}
			}
		}

		if (classifier.getOperationKind().equals("SIZE")
			|| classifier.getOperationKind().equals("SUM")) {
			OpKindEnum opkind = classifier.getOpKindEnum();
			parent.getVariableDeclaration().remove(opkind.getCallVar());
//			if (classifier.getOperationKind().equals("SUM")) {
//				parent.getVariableDeclaration().remove(enum.getIteratorVar());
//			}
		}
	}

	/**
	 * Returns if the ClassifierRoleSet is in edit modus or not.
	 * @return edit modus
	 */
	public boolean isEdit() {
		return isEdit;
	}

}
