package vocleditor.command;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.eclipse.swt.widgets.Shell;

import vocleditor.dialogs.CreateNewClassifierRoleSimpleDialog;
import vocleditor.model.Helpers;
import vocleditor.model.VOCLModelManager;
import VOCLModel.Attribute;
import VOCLModel.ClassifierRole;
import VOCLModel.Constraint;
import VOCLModel.Method;
import VOCLModel.NavExp;
import VOCLModel.VOCLExpression;
import VOCLModel.VOCLModelFactory;
import VOCLModel.VariableDeclaration;

/**
 * 
 * Class : CreateClassifierRoleSimpleCommand <br/>Package : vocleditor.command
 * <br/>Project : VOCLEditor <br/>Created On : 22.05.2004 <br/>Description :
 * Command for Creating a new ClassifierRole. <br/>
 * 
 * @author Gruppe 1: Stefan Hnsgen, Andre Hernich, Maximilian Kern, Sebastian
 *         Russin <br>
 */
public class CreateClassifierRoleSimpleCommand extends
		AbstractCreateClassifierRoleCommand {

	// Label to show in editor palette
	private static final String label = "Create Classifier Role";

	// Description shown in the editor
	private static final String description = "Creates a new classifier role";

	// Data for execute()
	private ClassifierRole classifierRole = null;

	private VariableDeclaration variableDeclaration = null;

	private Method methodDeclaration = null;

	private VariableDeclaration methodReturn = null;

	private List methodParameters = null;

	private Constraint constraint = null;

	private boolean isEdit = false;

	List attrList = new Vector();

	/**
	 * Creates the command.
	 */
	public CreateClassifierRoleSimpleCommand() {
		setLabel(label);
	}

	/**
	 * Creates the command and uses the given parent shell to display the dialog
	 * box.
	 * 
	 * @param shell
	 *            The shell.
	 */
	public CreateClassifierRoleSimpleCommand(Shell shell) {
		setLabel(label);
		this.shell = shell;
	}

	/**
	 * Set the current ClassifierRole.
	 * 
	 * @param classifierRole
	 */
	public void setClassifierRoleSimple(ClassifierRole classifierRole) {
		this.classifierRole = classifierRole;
	}

	/**
	 * Remove all modellelements of this classifierRoles if it is in edit modus.
	 * 
	 * @param classifier
	 */
	private void removeFromModel(ClassifierRole classifier) {
		//remove VariableDeclaration
		if (classifier.getClassifierVar() != null) {
			parent.getVariableDeclaration().remove(
					classifier.getClassifierVar());
		}
		//remove Method
		if (classifier.getMethod() != null) {
			if (classifier.getMethod().getReturns() != null) {
				parent.getVariableDeclaration().remove(
						classifier.getMethod().getReturns());
			}
			if (classifier.getMethod().getCall().size() > 0) {
				parent.getVariableDeclaration().removeAll(
						classifier.getMethod().getCall());
			}
		}

		//remove Attributes
		if (classifier.getAttribute().size() > 0) {
			List list = new Vector();
			for (Iterator it = classifier.getAttribute().iterator(); it
					.hasNext();) {
				Attribute attr = (Attribute) it.next();
				list.add(attr);
				if (attr.getVariableDeclaration() != null) {
					parent.getVariableDeclaration().remove(
							attr.getVariableDeclaration());
				}
			}
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {

		if (Helpers.constraintHasAlreadyVOCLExpression(shell, navexpParent) == 1) {
			return;
		}

		VOCLModelFactory mfc = VOCLModelManager.getFactory();

		//open a dialog for editing the settings
		CreateNewClassifierRoleSimpleDialog d = new CreateNewClassifierRoleSimpleDialog(
				shell);

		//if no navexp exists, create a new one
		if (parent == null && classifierRole.eContainer() == null) {
			parent = mfc.createNavExp();
			parent.setIndex(position);
			Helpers.addToParent(parent, navexpParent, position);
		} else if (parent == null && classifierRole.eContainer() != null) {
			parent = (NavExp) classifierRole.eContainer();
		}
		d.setParent(parent);
		//		if (isEdit) {
		//			d.setClassifierRole(classifierRole);
		//			removeFromModel(classifierRole);
		//		}
		//opens the dialog
		d.open();
		//		if (isEdit) {
		//			classifierRole.setMethod(null);
		//			classifierRole.setClassifierVar(null);
		//			classifierRole.getAttribute().clear();
		//		}
		if (d.isCanceled()) {
			Helpers.removeFromParent(navexpParent, position);
			return;
		}

		String type = d.getRoleTypeText();
		if (type == null)
			type = "";
		else
			type = type.trim();
		String varName = d.getRoleNameText();
		if (varName == null)
			varName = "";
		else
			varName = varName.trim();
		List attributes = d.getAttributes();

		// Set up classifier role
		classifierRole.setType(type);
		if (!varName.equals(""))
			classifierRole.setName(varName);

		//Create variable declaration
		if (!varName.equals("")) {
			variableDeclaration = mfc.createVariableDeclaration();
			variableDeclaration.setType(type);
			variableDeclaration.setVarName(varName);
			classifierRole.setClassifierVar(variableDeclaration);
		}

		// Create attributes
		if (attributes.size() > 0) {
			attrList = new Vector();
			for (Iterator it = attributes.iterator(); it.hasNext();) {
				Attribute attr = mfc.createAttribute();
				attr.setName((String) it.next());

				String attrVarType = (String) it.next();
				attr.setType(attrVarType);
				attr.setValue((String) it.next());
				String attrVarName = (String) it.next();
				//wenn eine Variablendeklaration da ist
				if (attrVarName != "") {
					VariableDeclaration letVar = Helpers
							.getLetExpVariablenDeclaration(parent);
					//berprfe, ob die Variablendeklaration in der LetExp vorkommt
					if (letVar != null) {
						if (letVar.getVarName().equals(attrVarName)) {
							attr.setVariableDeclaration(Helpers
									.getLetExpVariablenDeclaration(parent));
						} else {
							VariableDeclaration vardeclar = mfc
									.createVariableDeclaration();
							vardeclar.setType(attrVarType);
							vardeclar.setVarName(attrVarName);
							parent.getVariableDeclaration().add(vardeclar);
							attr.setVariableDeclaration(vardeclar);
						}

					} else {
						VariableDeclaration vardeclar = mfc
								.createVariableDeclaration();
						vardeclar.setType(attrVarType);
						vardeclar.setVarName(attrVarName);
						//Variablendeklaration zu NavExp hinzufgen
						parent.getVariableDeclaration().add(vardeclar);
						attr.setVariableDeclaration(vardeclar);
					}
				}

				classifierRole.getAttribute().add(attr);

				attrList.add(attr);
			}
		}

		if (d.getMethodName() != null) {
			methodDeclaration = mfc.createMethod();
			methodDeclaration.setName(d.getMethodName());
			if (d.getParameters().size() > 0) {
				methodParameters = new LinkedList();
				//Methodenparameter sammeln fr Einfgen ins Modell
				for (int i = 0; i < d.getParameters().size(); i += 2) {
					String pname = (String) d.getParameters().get(i);
					String ptype = (String) d.getParameters().get(i + 1);
					VariableDeclaration param = mfc.createVariableDeclaration();
					param.setVarName(pname);
					param.setType(ptype);
					methodParameters.add(param);
				}
			}
			if (!d.getResultName().equals("")) {
				methodReturn = mfc.createVariableDeclaration();
				methodReturn.setType(d.getResultType());
				methodReturn.setVarName(d.getResultName());

			}

		}
		insertIntoModel();
		if (loc != null) {
			classifierRole.setX(loc.x);
			classifierRole.setY(loc.y);
		}

	}

	/**
	 * Insert all data into the modell.
	 */
	private void insertIntoModel() {
		//insert the classifierRole into the modell
		parent.getClassifierNode().add(classifierRole);

		//insert the rolename and -type of the classifierRole into the modell
		if (variableDeclaration != null)
			parent.getVariableDeclaration().add(variableDeclaration);

		//insert the method into the modell
		classifierRole.setMethod(methodDeclaration);

		//insert the return - declaration of the method into the modell
		if (methodReturn != null) {
			parent.getVariableDeclaration().add(methodReturn);
			methodDeclaration.setReturns(methodReturn);
		}

		//insert the method parameters into the modell
		if (methodParameters != null) {
			parent.getVariableDeclaration().addAll(methodParameters);
			methodDeclaration.getCall().addAll(methodParameters);
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		if (parent.getClassifierNode().size() == 0) {
			Helpers.addToParent(parent, (VOCLExpression) navexpParent, parent
					.getIndex());
		}
		if (attrList.size() > 0) {
			for (Iterator it = attrList.iterator(); it.hasNext();) {
				Attribute attribute = (Attribute) it.next();
				if (attribute.getVariableDeclaration() == null
						&& attribute.getValue().equals("")) {
					attribute.setVariableDeclaration(Helpers
							.getLetExpVariablenDeclaration(parent));
				} else if (attribute.getVariableDeclaration() != null) {
					parent.getVariableDeclaration().add(
							attribute.getVariableDeclaration());

				}
			}
		}

		insertIntoModel();

	}

	public void undo() {
		parent.getClassifierNode().remove(classifierRole);
		if (variableDeclaration != null)
			parent.getVariableDeclaration().remove(variableDeclaration);

		if (attrList.size() > 0) {
			for (Iterator it = attrList.iterator(); it.hasNext();) {
				Attribute attribute = (Attribute) it.next();
				if (attribute.getVariableDeclaration() != null) {
					if (attribute.getVariableDeclaration() == Helpers
							.getLetExpVariablenDeclaration(parent)) {
						attribute.setVariableDeclaration(null);
					} else {
						parent.getVariableDeclaration().remove(
								attribute.getVariableDeclaration());
					}
				}
			}
		}

		if (methodReturn != null) {
			parent.getVariableDeclaration().remove(methodReturn);
			//			methodDeclaration.setReturns(methodReturn);
		}
		if (methodParameters != null) {
			parent.getVariableDeclaration().removeAll(methodParameters);
			methodDeclaration.getCall().removeAll(methodParameters);
		}

		if (parent.getClassifierNode().size() == 0) {
			Object navexpParent = parent.eContainer();
			Helpers.removeFromParent(navexpParent, parent.getIndex());
		}
	}

	/**
	 * Indicates whether this classifierRole is in edit modus or not.
	 * 
	 * @param b
	 */
	public void setEdit(boolean b) {
		isEdit = b;
	}

}