package vocleditor.command;

import java.awt.Point;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import vocleditor.model.Helpers;
import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationRole;
import VOCLModel.Attribute;
import VOCLModel.ClassifierNode;
import VOCLModel.ClassifierRole;
import VOCLModel.ClassifierRoleSet;
import VOCLModel.Condition;
import VOCLModel.Method;
import VOCLModel.NavExp;
import VOCLModel.OpKindEnum;
import VOCLModel.VariableDeclaration;

/**
 * 
 * Class       : DeleteClassifierRoleCommand<br/>
 * Package     : vocleditor.command<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 17.05.2004<br/>
 * Description : Command for deleting ClassifierNodes.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class DeleteClassifierRoleCommand
	extends org.eclipse.gef.commands.Command {
	private static final String DeleteCommand_Label = "Delete ClassifierNode";
	private ClassifierNode classifierRole;
	private VariableDeclaration varDec;
	private Point loc;
	private EList params;
	private VariableDeclaration returns;
	private VariableDeclaration iteratorVar = null;
	private List attrVars;
	private Method method;
	private NavExp parent;
	private Object navexpParent;

	private int index = -1;
	private List associationRoles;
	Hashtable sourceClassifier;
	Hashtable targetClassifier;

	List attributeVarDec = new LinkedList();
	VariableDeclaration callVar = null;

	Shell shell;

	/**
	 * 
	 */
	public DeleteClassifierRoleCommand(Shell shell) {
		super(DeleteCommand_Label);
		this.shell = shell;
	}

	/**
	 * @see org.eclipse.gef.commands.Command#getLabel()
	 */
	public String getLabel() {
		return DeleteCommand_Label;
	}

	private void deleteConnections(ClassifierNode classifierRole) {
		ListIterator li = parent.getAssociationRole().listIterator();

		while (li.hasNext()) {
			AssociationRole associationRole = (AssociationRole) li.next();
			AssociationEndRoleTarget associationEndRoleTarget =
				associationRole.getAssociationEndRoleTarget();
			AssociationEndRoleSource associationEndRoleSource =
				associationRole.getAssociationEndRoleSource();

			ClassifierNode targetClassifierRole =
				associationEndRoleTarget.getClassifierNode();
			ClassifierNode sourceClassifierRole =
				associationEndRoleSource.getClassifierNode();

			if (targetClassifierRole == classifierRole) {
				sourceClassifier.put(
					associationEndRoleSource,
					sourceClassifierRole);
				sourceClassifierRole.getAssociationEndRole().remove(
					associationEndRoleSource);
				associationRoles.add(associationRole);
			}
			if (sourceClassifierRole == classifierRole) {
				targetClassifier.put(
					associationEndRoleTarget,
					targetClassifierRole);
				targetClassifierRole.getAssociationEndRole().remove(
					associationEndRoleTarget);
				associationRoles.add(associationRole);
			}
		}
		li = associationRoles.listIterator();
		while (li.hasNext()) {
			parent.getAssociationRole().remove(li.next());
		}

	}

	/**
	 * Help method that checks whether the ClassifierNode has
	 * a Variable that is used in a Condition.
	 * @param condition
	 * @return
	 */
	private boolean checkHasCondition(Condition condition) {
		boolean temp = false;

		if (condition != null) {
			VariableDeclaration v = condition.getLeft();
			//Checks if a Variable of an attribute is used in this condition
			List attrList = classifierRole.getAttribute();
			if (attrList.size() > 0) {
				Iterator it = attrList.iterator();
				while (it.hasNext()) {
					Attribute attr = (Attribute) it.next();
					if (attr != null) {
						if (attr.getVariableDeclaration() == v) {

							temp = true;
						}
					}
				}
			}

			v = condition.getRight();
			if (v != null) {
				if (attrList.size() > 0) {
					Iterator it = attrList.iterator();
					while (it.hasNext()) {
						Attribute attr = (Attribute) it.next();
						if (attr != null) {
							if (attr.getVariableDeclaration() == v) {

								temp = true;
							}
						}
					}
				}
			}

			if (classifierRole instanceof ClassifierRole) {
				//check if the variable of the method is used in the condition
				Method method = ((ClassifierRole) classifierRole).getMethod();
				if (method != null) {
					VariableDeclaration result = method.getReturns();
					if (result != null) {
						if (v == result) {
							temp = true;
						}
					}
				}
			}
				
			if (classifierRole instanceof ClassifierRoleSet) {
				OpKindEnum kindEnum =
					((ClassifierRoleSet) classifierRole).getOpKindEnum();
				//checks if a variable of a size / sum operation is used in this condition	
				if (kindEnum != null) {
					callVar = kindEnum.getCallVar();
					if (callVar == v) {
						temp = true;
					}
				}
			}
			v = condition.getLeft();
			if (classifierRole instanceof ClassifierRoleSet) {
				OpKindEnum kindEnum =
					((ClassifierRoleSet) classifierRole).getOpKindEnum();
				//checks if a variable of a size / sum operation is used in this condition	
				if (kindEnum != null) {
					callVar = kindEnum.getCallVar();
					if (callVar == v) {
						temp = true;
					}
				}
			}
			if (classifierRole instanceof ClassifierRole) {
				//check if the variable of the method is used in the condition
				Method method = ((ClassifierRole) classifierRole).getMethod();
				if (method != null) {
					VariableDeclaration result = method.getReturns();
					if (result != null) {
						if (v == result) {
							temp = true;
						}
					}
				}
			}
		}
		return temp;
	}

	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		boolean beenden = false;
		sourceClassifier = new Hashtable();
		targetClassifier = new Hashtable();
		associationRoles = new Vector();
		parent = classifierRole.getNavExp();

		beenden =
			checkHasCondition(parent.getCondition())
				|| checkHasCondition(
					Helpers.searchConstraint(parent).getCondition());
		if (beenden) {
			MessageBox msg = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
			msg.setText("Error");
			msg.setMessage(
				"A variable of this ClassifierRole is used by a condition!!\n"
					+ "Delete this condition first and then the ClassifierRole!");
			msg.open();
			return;
		}
		deleteConnections(classifierRole);
		//VariableDeclaration der ClassifierRole
		varDec = classifierRole.getClassifierVar();

		attrVars = classifierRole.getAttribute();
		if (attrVars != null) {
			Iterator it = attrVars.iterator();
			while (it.hasNext()) {
				Attribute attrs = (Attribute) it.next();
				if (attrs.getVariableDeclaration() != null) {
					if (attrs.getVariableDeclaration()
						== Helpers.getLetExpVariablenDeclaration(parent)) {
						attrs.setVariableDeclaration(null);
					} else {
						attributeVarDec.add(attrs.getVariableDeclaration());
					}
				}
			}
		}

		if (classifierRole instanceof ClassifierRoleSet) {
			OpKindEnum kindEnum =
				((ClassifierRoleSet) classifierRole).getOpKindEnum();
			if (kindEnum != null) {

				iteratorVar = kindEnum.getIteratorVar();
				callVar = kindEnum.getCallVar();

			}
		}

		//Attribute, die auf eine VariableDeclaration zeigen, entfernen
		if (classifierRole instanceof ClassifierRole) {

			method = ((ClassifierRole) classifierRole).getMethod();
			if (method != null) {
				returns = method.getReturns();
				params = method.getCall();

			}
		}
		removeFromModel();
	}

	/**
	 * Removes all Elements referenced by a classifierRoleNode and the ClassifierRole itself.
	 */
	private void removeFromModel() {

		if (varDec != null) {
			parent.getVariableDeclaration().remove(varDec);
		}

		//remove ClassifierRoleNode from the modell
		parent.getClassifierNode().remove(classifierRole);

		//remove the method of a ClassifierRole from the modell
		if (method != null) {
			parent.getVariableDeclaration().removeAll(params);
			parent.getVariableDeclaration().remove(returns);
		}

		//remove callVar if a OpKindEnum exists
		if (callVar != null) {
			parent.getVariableDeclaration().remove(callVar);
		}

		//remove iteratorVar if a OpKindEnum exists (SUM-Function)
		if (iteratorVar != null) {
			parent.getVariableDeclaration().remove(iteratorVar);
		}

		//remove all attributes of a ClassifierRoleNode
		if (attributeVarDec.size() > 0) {
			parent.getVariableDeclaration().removeAll(attributeVarDec);
		}

		//if there is not a ClassifierNode on the navexp, delete also the navexp
		if (parent.getClassifierNode().size() == 0
			&& parent.getCondition() == null) {
			navexpParent = parent.eContainer();
			Helpers.removeFromParent(navexpParent, parent.getIndex());
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		deleteConnections(classifierRole);
		removeFromModel();
	}

	/**
	 * @param classifierRole
	 * sets the current ClassifierRoleNode
	 */
	public void setClassifierRole(ClassifierNode classifierRole) {
		this.classifierRole = classifierRole;
	}

	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {

		if (parent.getClassifierNode().size() == 0) {
			Helpers.addToParent(parent, navexpParent, parent.getIndex());
		}
		for (Iterator it = classifierRole.getAttribute().iterator();
			it.hasNext();
			) {
			Attribute attr = (Attribute) it.next();
			if (attr.getVariableDeclaration() == null
				&& attr.getValue().equals("")) {
				attr.setVariableDeclaration(
					Helpers.getLetExpVariablenDeclaration(parent));
			}
		}
		ListIterator li = associationRoles.listIterator();
		while (li.hasNext()) {
			AssociationRole associationRole = (AssociationRole) li.next();
			AssociationEndRoleSource associationEndRoleSource =
				associationRole.getAssociationEndRoleSource();
			AssociationEndRoleTarget associationEndRoleTarget =
				associationRole.getAssociationEndRoleTarget();
			ClassifierNode sourceClassifierRole =
				associationEndRoleSource.getClassifierNode();
			ClassifierNode targetClassifierRole =
				associationEndRoleTarget.getClassifierNode();
			if (targetClassifierRole == classifierRole) {
				sourceClassifierRole =
					(ClassifierNode) sourceClassifier.get(
						associationEndRoleSource);
				sourceClassifierRole.getAssociationEndRole().add(
					associationEndRoleSource);
			}
			if (sourceClassifierRole == classifierRole) {
				targetClassifierRole =
					(ClassifierNode) targetClassifier.get(
						associationEndRoleTarget);
				targetClassifierRole.getAssociationEndRole().add(
					associationEndRoleTarget);
			}
			parent.getAssociationRole().add(associationRole);
		}

		//if classiferRoleNode has VariableDeclaration add it again
		if (varDec != null) {
			parent.getVariableDeclaration().add(varDec);
		}
		//insert classifierRoleNode into modell again
		parent.getClassifierNode().add(classifierRole);

		//insert method into modell again
		if (method != null) {
			((ClassifierRole) classifierRole).setMethod(method);
			method.setReturns(returns);
			method.getCall().addAll(params);
			parent.getVariableDeclaration().addAll(params);
			parent.getVariableDeclaration().add(returns);
		}

		//insert all attributes into modell again
		if (attributeVarDec.size() > 0) {
			parent.getVariableDeclaration().addAll(attributeVarDec);
		}

		if (callVar != null)
			parent.getVariableDeclaration().add(callVar);

		if (iteratorVar != null) {
			parent.getVariableDeclaration().add(iteratorVar);
		}

	}

}
