package vocleditor.command;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;

import vocleditor.model.Helpers;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.NavExp;
import VOCLModel.VOCLExpression;
import VOCLModel.VariableDeclaration;


/**
 * 
 * Class       : DeleteConditionCommand<br/>
 * Package     : vocleditor.command<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 14.06.2004<br/>
 * Description : Command for deleting Conditions.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class DeleteConditionCommand extends Command {

	//the current condition
	Condition condition;
	//the variable on the left-side
	VariableDeclaration left;
	//the variable on the right-side
	VariableDeclaration right = null;
	//the parent (Navexp, Constraint) of the condition
	EObject parent;
	

	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() {
		left = condition.getLeft();
		//if right is a VariableDeclaration
		if (condition.getRight() != null) {
			right = condition.getRight();
		}
		condition.setLeft(null);
		condition.setRight(null);
		//delete referenec from the parent
		if(parent instanceof NavExp){
			((NavExp)parent).setCondition(null);
						if(((NavExp)parent).getClassifierNode().size() <= 0){
							Helpers.removeFromParent(parent,((NavExp)parent).getIndex());
						}
		}else
			((Constraint)parent).setCondition(null);
	}

	/**
	 * Sets the current Condition.
	 * @param c the current Condition
	 */
	public void setCondition(Condition c) {
		condition = c;
		//set the parent of this condition
		parent = condition.eContainer();
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() {
		condition.setLeft(left);
		if(parent instanceof VOCLExpression){
			((NavExp)parent).setCondition(condition);
			if(((NavExp)parent).getClassifierNode().size()<=0){
				Helpers.addToParent((NavExp)parent,parent.eContainer(),((NavExp)parent).getIndex());	
			}
		}else
			((Constraint)parent).setCondition(condition);
		if(right != null){
			condition.setRight(right);
		}
	}

	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() {
		execute();
	}


}
