package vocleditor.command;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.commands.Command;

import VOCLModel.ClassifierNode;

/**
 * 
 * Class       : SetClassifierConstraintCommand<br/>
 * Package     : vocleditor.command<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 14.06.2004<br/>
 * Description : Command for moving ClassfierNodes.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class SetClassifierConstraintCommand extends Command
{
	private static final String locationLabel = "change location command";
	private static final String resizeLabel = "resize command";
	private Point newPos;
	private Point oldPos;
	private ClassifierNode part;
	
	/**
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	public void execute() 
	{
		oldPos = new Point(part.getX(), part.getY());
		part.setX(newPos.x);
		part.setY(newPos.y);
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#getLabel()
	 */
	public String getLabel() {
		return resizeLabel;
	}

	
	/**
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	public void redo() 
	{
		part.setX(newPos.x);
		part.setY(newPos.y);
	}
	
	/**
	 * Sets the current Coordinates of the ClassifierNode.
	 * @param r - the constraint
	 */
	public void setLocation(Rectangle r)
	{
		newPos = new Point(r.x,r.y);
	}
	
	/**
	 * Sets the current ClassifierNode.
	 * @param part
	 */
	public void setPart(ClassifierNode part) 
	{
		this.part = part;
	}
	
	/**
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	public void undo() 
	{
		part.setX(oldPos.x);
		part.setY(oldPos.y);
	}
}
