
package vocleditor.edit.parts;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ManhattanConnectionRouter;
import org.eclipse.draw2d.RelativeBendpoint;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import org.eclipse.swt.widgets.Shell;

import vocleditor.edit.AssociationRoleEditPolicy;
import vocleditor.edit.policies.AssociationRoleBendPointEditPolicy;
import vocleditor.edit.policies.AssociationRoleEndpointEditPolicy;
import vocleditor.figures.AssociationRoleFigure;
import VOCLModel.AssociationRole;
import VOCLModel.BendPoint;
import VOCLModel.VOCLModelPackage;

/**
 * Class       : AssociationRoleEditPart<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This editpart represents an association role <br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class AssociationRoleEditPart extends AbstractConnectionEditPart {

	private AssociationRoleAdapter adapter = new AssociationRoleAdapter();
	private Shell shell;

	/**
	 * @return Returns the shell.
	 */
	protected Shell getShell() {
		return shell;
	}
	/**
	 * @param shell The shell to set.
	 */
	protected void setShell(Shell shell) {
		this.shell = shell;
	}
	public AssociationRoleEditPart(AssociationRole arc) {
		super();
		setModel(arc);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(
			EditPolicy.CONNECTION_ENDPOINTS_ROLE,
			new AssociationRoleEndpointEditPolicy());
		installEditPolicy(
				EditPolicy.CONNECTION_BENDPOINTS_ROLE,
				new AssociationRoleBendPointEditPolicy());
		installEditPolicy(
			EditPolicy.COMPONENT_ROLE, 
			new AssociationRoleEditPolicy());
	}

	public AssociationRole getAssociationRole() {
		return (AssociationRole) getModel();
	}

	private class AssociationRoleAdapter implements Adapter {
		Notifier newTarget;

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
		 */
		public Notifier getTarget() {
			return newTarget;
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
		 */
		public boolean isAdapterForType(Object type) {
			return getModel().getClass() == type;
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification notification) {

			int featureId = notification.getFeatureID(VOCLModelPackage.class);

			switch (featureId) {
				case VOCLModelPackage.ASSOCIATION_ROLE__BEND_POINT :
					refreshBendpoints();
					break;

			}

		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
		public void setTarget(Notifier newTarget) {
			this.newTarget = newTarget;
		}
	}

	/**
		  * @see org.eclipse.gef.EditPart#activate()
		  */
	public void activate() {
		getAssociationRole().eAdapters().add(adapter);
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	public void deactivate() {
		getAssociationRole().eAdapters().remove(adapter);
		super.deactivate();
	}
	/**
		* @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
		*/
	protected IFigure createFigure() {
		AssociationRoleFigure f = new AssociationRoleFigure();
		return f;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		Vector children = new Vector();
		children.add(getAssociationRole().getAssociationEndRoleTarget());
		children.add(getAssociationRole().getAssociationEndRoleSource());
		return children;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		refreshBendpoints();
		super.refreshVisuals();
	}

	/**
	 * Updates the bendpoints, based on the model.
	 */
	protected void refreshBendpoints() {
		if (getConnectionFigure().getConnectionRouter()
			instanceof ManhattanConnectionRouter)
			return;
		List modelConstraint = getAssociationRole().getBendPoint();
		List figureConstraint = new ArrayList();
		if (modelConstraint != null) {

			for (int i = 0; i < modelConstraint.size(); i++) {
				BendPoint wbp = (BendPoint) modelConstraint.get(i);
				RelativeBendpoint rbp =
					new RelativeBendpoint(getConnectionFigure());
				Dimension first =
					new Dimension(
						((Integer) wbp.getW().get(0)).intValue(),
						((Integer) wbp.getH().get(0)).intValue());
				Dimension second =
					new Dimension(
						((Integer) wbp.getW().get(1)).intValue(),
						((Integer) wbp.getH().get(1)).intValue());
				rbp.setRelativeDimensions(first, second);
				figureConstraint.add(rbp);
			}

			getConnectionFigure().setRoutingConstraint(figureConstraint);
		}
	}
}
