package vocleditor.edit.parts;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policies.ClassifierComponentEditPolicy;
import vocleditor.edit.policies.ClassifierRoleSetEditPolicy;
import vocleditor.figures.SetClassifierRoleFigure;
import vocleditor.model.EObjectPropertySource;
import vocleditor.model.IAnchorFigure;
import vocleditor.model.NavExpConstants;
import vocleditor.model.NavExpEditorAnchor;
import VOCLModel.AssociationEndRole;
import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationRole;
import VOCLModel.Attribute;
import VOCLModel.ClassifierRoleSet;
import VOCLModel.ModelElement;
import VOCLModel.OpKindEnum;
import VOCLModel.VOCLModelPackage;
import VOCLModel.VariableDeclaration;

/**
 * Class       : ClassifierRoleSetEditPart<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This editpart represents a set classifier role<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class ClassifierRoleSetEditPart
	extends AbstractClassifierRole {

	/**
	 * Creates a new ClassifierRoleEditPart instance for the given classifier
	 * role model.
	 * 
	 * @param classifierRole The classifier role model.
	 */
	public ClassifierRoleSetEditPart(ClassifierRoleSet classifierRole) {
		setModel(classifierRole);
	}

	/**
	 * Returns the classifierRole model of the edit part.
	 */
	public ClassifierRoleSet getClassifierRoleSet() {
		return (ClassifierRoleSet) getModel();
	}

	/**
	 * Creates the figure.
	 * 
	 * @return The figure.
	 */
	protected IFigure createFigure() {
		SetClassifierRoleFigure figure = new SetClassifierRoleFigure();
		return figure;
	}

	/**
	 * Returns the classifierRoleSet figure.
	 */
	protected SetClassifierRoleFigure getClassifierRoleFigure() {
		return (SetClassifierRoleFigure) getFigure();
	}

	/**
	 * Creates edit policies.
	 */
	protected void createEditPolicies() {
		installEditPolicy(
			EditPolicy.GRAPHICAL_NODE_ROLE,
			new ClassifierRoleSetEditPolicy());
		installEditPolicy(
			EditPolicy.COMPONENT_ROLE,
			new ClassifierComponentEditPolicy());

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 * eigene implementierung
	 */
	protected List getModelSourceConnections() {
		List list = new LinkedList();
		Iterator it;
		it = getClassifierRoleSet().getAssociationEndRole().iterator();
		while (it.hasNext()) {
			AssociationEndRole endRole = (AssociationEndRole) it.next();
			if (endRole instanceof AssociationEndRoleSource)
				list.add(
					((AssociationEndRoleSource) endRole).getAssociationRole());

		}
		return list;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 * eigene implementierung
	 */
	protected List getModelTargetConnections() {
		List list = new LinkedList();
		Iterator it;
		it = getClassifierRoleSet().getAssociationEndRole().iterator();
		while (it.hasNext()) {
			AssociationEndRole endRole = (AssociationEndRole) it.next();
			if (endRole instanceof AssociationEndRoleTarget)
				list.add(
					((AssociationEndRoleTarget) endRole).getAssociationRole());
		}
		return list;
	}


	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		NavExpEditorAnchor.attachAndRefreshAnchors((IAnchorFigure) getFigure());
		AssociationRole asso =
			((AssociationRoleEditPart) connection)
				.getAssociationRole()
				.getAssociationEndRoleSource()
				.getAssociationRole();
		int id = asso.getSourceAnchorId();
		return (ConnectionAnchor) getClassifierRoleFigure()
			.getSourceAnchorList()
			.get(
			id);
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		NavExpEditorAnchor.attachAndRefreshAnchors((IAnchorFigure) getFigure());
		AssociationRole asso = null;
		if (request instanceof CreateConnectionRequest) {
			asso =
				(AssociationRole) (((CreateConnectionRequest) request)
					.getNewObject());
		}
		if (request instanceof ReconnectRequest) {
			asso =
				(AssociationRole)
					((AssociationRoleEditPart) ((ReconnectRequest) request)
					.getConnectionEditPart())
					.getAssociationRole();
		}
		ConnectionAnchor anchor =
			getClassifierRoleFigure().getSourceConnectionAnchor();
		asso.setSourceAnchorId(
			getClassifierRoleFigure().getSourceAnchorList().indexOf(anchor));
		return anchor;
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		NavExpEditorAnchor.attachAndRefreshAnchors((IAnchorFigure) getFigure());
		AssociationRole asso =
			((AssociationRoleEditPart) connection)
				.getAssociationRole()
				.getAssociationEndRoleTarget()
				.getAssociationRole();
		int id = asso.getTargetAnchorId();
		return (ConnectionAnchor) getClassifierRoleFigure()
			.getTargetAnchorList()
			.get(
			id);
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		NavExpEditorAnchor.attachAndRefreshAnchors((IAnchorFigure) getFigure());
		AssociationRole asso = null;
		if (request instanceof CreateConnectionRequest) {
			asso =
				(AssociationRole) (((CreateConnectionRequest) request)
					.getNewObject());
		}
		if (request instanceof ReconnectRequest) {
			asso =
				(AssociationRole)
					((AssociationRoleEditPart) ((ReconnectRequest) request)
					.getConnectionEditPart())
					.getAssociationRole();
		}
		ConnectionAnchor anchor =
			getClassifierRoleFigure().getTargetConnectionAnchor();
		asso.setTargetAnchorId(
			getClassifierRoleFigure().getTargetAnchorList().indexOf(anchor));
		return anchor;
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate() {
		if (isActive())
			return;

		hookIntoModelElement(getClassifierRoleSet());

		Iterator it;

		it = getClassifierRoleSet().getAssociationEndRole().iterator();
		while (it.hasNext()) {
			AssociationEndRole assER = (AssociationEndRole) it.next();
			hookIntoModelElement(assER);
		}
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;

		//((Notifier)getClassifierRole()).eAdapters().remove(this);

		unhookFromModelElement(getClassifierRoleSet());
		super.deactivate();

		Iterator it;
		it = getClassifierRoleSet().getAssociationEndRole().iterator();
		while (it.hasNext()) {
			AssociationEndRole assER = (AssociationEndRole) it.next();
			unhookFromModelElement(assER);
		}

	}

	/**
	 * Catch notifications for model changes.
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(VOCLModelPackage.class);
		switch (featureId) {
			case VOCLModelPackage.CLASSIFIER_ROLE_SET__ASSOCIATION_END_ROLE :
				AssociationEndRole endRole =
					(AssociationEndRole) notification.getNewValue();
				//				if (endRole instanceof AssociationEndRoleSource) {
				refreshSourceConnections();
				//				} else {
				refreshTargetConnections();
				//				}
				break;

			case VOCLModelPackage.CLASSIFIER_ROLE_SET :
				refreshVisuals();
				break;
			default :
				refreshVisuals();
		}
	}

	/**
		* Registers this edit part as a listener for change notifications
		* to the specified modelElement.
		* 
		* @param element the NexExp element that should be observed
		* for change notifications
		*/
	protected void hookIntoModelElement(ModelElement element) {
		if (element != null)
			element.eAdapters().add(this);
	}
	/**
	  * Removes this edit part from the specified NavExp element.
	  * Thus, it will no longe receive change notifications.
	  * 
	  * @param element the NavExp element that should not be observed
	  * any more
	  */
	protected void unhookFromModelElement(ModelElement element) {
		if (element != null)
			element.eAdapters().remove(this);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		ClassifierRoleSet role = getClassifierRoleSet();
		SetClassifierRoleFigure figure = getClassifierRoleFigure();

		String type = role.getType();
		if (type != null)
			figure.setClass(type);
		else
			figure.setClass("");

		// Set name
		VariableDeclaration varDecl = role.getClassifierVar();
		if (varDecl != null) {
			String name = varDecl.getVarName();
			if (name != null)
				figure.setVariable(varDecl.getVarName());
			else
				figure.setVariable("");
		} else
			figure.setVariable("");

		//Set operationKind ...
		String operationKind = role.getOperationKind();
		figure.setOperationType(operationKind);
		//EVAL DIRTY HACK ;-) ...
		OpKindEnum opkindenum;
		VariableDeclaration callVar = null;
		VariableDeclaration iteratorVar = null;
		if (operationKind.equals(NavExpConstants.SIZE)
			|| operationKind.equals(NavExpConstants.SUM)) {
			opkindenum = (OpKindEnum) role.getOpKindEnum();
			if (opkindenum != null) {
				callVar = opkindenum.getCallVar();
				iteratorVar = opkindenum.getIteratorVar();
			}
		}

		String opvar;
		if (iteratorVar != null) {
			if (operationKind.equals(NavExpConstants.SUM)) {
				opvar =
					"("
						+ iteratorVar.getVarName()
						+ ")="
						+ callVar.getVarName();
				figure.setOpVarName(opvar);
			}
		}
		if (operationKind.equals(NavExpConstants.SIZE)) {
			if (callVar != null) {
				opvar = callVar.getVarName();
				figure.setOpVarName(opvar);
			}
		}

		//Set Attribute
		//		Set attributes
		List attributes = role.getAttribute();
		String attributeString = "";
		boolean first = true;
		for (Iterator it = attributes.iterator(); it.hasNext();) {
			Attribute attr = (Attribute) it.next();
			if (attr != null) {
				if (!first)
					attributeString += "\n";
				else
					first = false;

				String attrName = attr.getName();
				if (attrName == null)
					attrName = "";
				String attrType = attr.getType();
				if (type == null)
					attrType = "";
				String value = attr.getValue();
				VariableDeclaration v = attr.getVariableDeclaration();
				if (value == null)
					value = "";
				if (v != null && value.equals("")) {
					value = v.getVarName();
				}
				attributeString += attr.getName() + ":" + attr.getType();
				if (!value.equals(""))
					attributeString += " = " + value;
			}
		}
		figure.setAttrName(attributeString);

		// Graphical properties
		Point loc =
			new Point(
				getClassifierRoleSet().getX(),
				getClassifierRoleSet().getY());
		Dimension size = new Dimension(-1, -1);
		Rectangle r = new Rectangle(loc, size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			r);

	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key)
			return getPropertySource();
		return super.getAdapter(key);
	}

	/**
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null)
			propertySource = new EObjectPropertySource(getClassifierRoleSet());
		return propertySource;
	}

}
