
package vocleditor.edit.parts;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policies.VOCLExpLayoutEditPolicy;
import vocleditor.figures.DecisionFigure;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.IfExp;

/**
 * Class       : DecisionEditPart<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This editpart represents a if-then-else expression<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 *
 */
public class DecisionEditPart
	extends VOCLExpEditPart {

	Notifier target;
	private IPropertySource propertySource = null;


	public DecisionEditPart(IfExp exp) {
		setModel(exp);
	}
	/**
		 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
		 */
	public void activate() {
		if (isActive())
			return;
		((Notifier) getDecisionExp()).eAdapters().add(this);
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;
		((Notifier) getDecisionExp()).eAdapters().remove(this);
		super.deactivate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		DecisionFigure figure = new DecisionFigure();
		figure.setIndex(getDecisionExp().getIndex());
		return figure;
	}
	/**
	 * Returns the Figure represented by this edit part
	 * @return
	 */
	public DecisionFigure getDecisionFigure() {
		return (DecisionFigure) getFigure();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(
			EditPolicy.LAYOUT_ROLE,
			new VOCLExpLayoutEditPolicy());
		super.createEditPolicies();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		List children = new LinkedList();
		if (getDecisionExp().getIfCondition() != null) {
			children.add(getDecisionExp().getIfCondition());
		}

		if (getDecisionExp().getElseExp() != null) {
			children.add(getDecisionExp().getElseExp());
		}

		if (getDecisionExp().getThenExp() != null) {
			children.add(getDecisionExp().getThenExp());
		}

		return children;
	}
	/**
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null)
			propertySource = new EObjectPropertySource(getDecisionExp());
		return propertySource;
	}

	public IfExp getDecisionExp() {
		return (IfExp) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		IfExp bool = getDecisionExp();
		DecisionFigure figure = getDecisionFigure();
		super.refreshVisuals();
	}

}
