
package vocleditor.edit.parts;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.policies.VOCLExpLayoutEditPolicy;
import vocleditor.figures.LetExpFigure;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.LetExp;

/**
 * Class       : LetExpEditPart<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This editpart represents a let-in expression<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class LetExpEditPart extends VOCLExpEditPart {

	LetExp letExp;
	Notifier target;
	private IPropertySource propertySource = null;


	/**
	 * Constructs a new LetExpEditPart
	 */
	public LetExpEditPart(LetExp letExp) {
		setModel(letExp);
		this.letExp = letExp;
	}

	/**
		 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
		 */
	public void activate() {
		if (isActive())
			return;
		((Notifier)getLetExp()).eAdapters().add(this);		
		
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;
		((Notifier) getLetExp()).eAdapters().remove(this);
		super.deactivate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		LetExpFigure figure = new LetExpFigure();
		figure.setIndex(getLetExp().getIndex());
		return figure;
	}

	/**
	 * Returns the LetExpFigure represented by this edit part
	 * 
	 * @return
	 */
	public LetExpFigure getLetExpFigure() {
		return (LetExpFigure) getFigure();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();
		installEditPolicy(
			EditPolicy.LAYOUT_ROLE,
			new VOCLExpLayoutEditPolicy());

	}

	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		List children = new LinkedList();
		if (getLetExp().getDef() != null)
			children.add(getLetExp().getDef());
		if (getLetExp().getInExp() != null)
			children.add(getLetExp().getInExp());
		return children;
	}
	/**
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null)
			propertySource = new EObjectPropertySource(getLetExp());
		return propertySource;
	}

	public LetExp getLetExp() {
		return (LetExp) getModel();
	}

	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		LetExp bool = getLetExp();
		LetExpFigure figure = getLetExpFigure();
		figure.setVertical(bool.isIsVertical());
		figure.setNegated(bool.isIsNegated());
		figure.setVariable(bool.getVariableDeclaration().getVarName()+" : "+bool.getVariableDeclaration().getType());
		super.refreshVisuals();
	}


}
