package vocleditor.edit.parts;


import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.swt.widgets.Shell;

import vocleditor.edit.policies.VOCLExpComponentEditPolicy;
import vocleditor.figures.VOCLExpFigure;
import VOCLModel.VOCLExpression;

/**
 * Class       : VOCLExpEditPart<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This editpart represents VOCL expression (obtain further information by watching the model)<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class VOCLExpEditPart extends AbstractGraphicalEditPart implements Adapter{
	
	private Shell shell;
	private Notifier target;
	
	/**
	 * Method setShell
	 * @return
	 */	
	public void setShell(Shell shell){
		this.shell = shell;
	}
	
	/**
	 * Method getShell
	 * @return
	 */	
	public Shell getShell(){
		return shell;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		return null;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,new VOCLExpComponentEditPolicy());
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */	
	protected void refreshVisuals() {
		IFigure figure = getFigure();
		Object o = getModel();
		if (o instanceof VOCLExpression && figure instanceof VOCLExpFigure){
			((VOCLExpFigure)getFigure()).setNegated(((VOCLExpression)getModel()).isIsNegated());
		}
		super.refreshVisuals();
			
	}

	/**
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
	public void notifyChanged(Notification notification) {
//		refresh();
		refreshChildren();
		refreshVisuals();
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/**
		 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
		 */
	public void setTarget(Notifier target) {
		this.target = target;
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}
}
