package vocleditor.edit.policies;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;

import vocleditor.command.MoveAssociationEndRoleCommand;
import vocleditor.edit.parts.AssociationRoleEditPart;
import VOCLModel.AssociationEndRole;

/**
 * Class       : AssociationEndRoleMovePolicy<br/>
 * Package     : vocleditor.edit.policies<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This policy is for moving the role label<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class AssociationEndRoleMovePolicy extends NonResizableEditPolicy
{
	/**
	 * Returns the command to move the end role label.
	 */
	public Command getMoveCommand(ChangeBoundsRequest request)
	{
		AssociationEndRole model = (AssociationEndRole)getHost().getModel();
		Point delta = request.getMoveDelta();
		MoveAssociationEndRoleCommand command = new MoveAssociationEndRoleCommand(model, getParentFigure(), delta);
		return command;
	}

	/**
	 * Returns the parent figure (the association role figure)
	 * 
	 * @return
	 */
	public Figure getParentFigure()
	{
		AssociationRoleEditPart edge = (AssociationRoleEditPart)getHost().getParent();
		return (Figure)edge.getFigure();
	}

}