// File: PetriNetXYLayoutEditPolicy.java
// Adapted from PetriNetXYLayoutEditPolicy.java

package vocleditor.edit.policies;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.swt.widgets.Shell;

import vocleditor.command.CreateClassifierRoleSetCommand;
import vocleditor.command.CreateClassifierRoleSimpleCommand;
import vocleditor.command.CreateConditionCommand;
import vocleditor.command.SetClassifierConstraintCommand;
import vocleditor.edit.parts.AssociationRoleEditPart;
import vocleditor.edit.parts.ClassifierRoleSetEditPart;
import vocleditor.edit.parts.ClassifierRoleSimpleEditPart;
import vocleditor.edit.parts.NavExpEditPart;
import vocleditor.model.ModelCreationFactory;
import VOCLModel.ClassifierNode;
import VOCLModel.ClassifierRole;
import VOCLModel.ClassifierRoleSet;
import VOCLModel.Condition;
import VOCLModel.NavExp;
/**
 * Class       : NavExpXYLayoutEditPolicy<br/>
 * Package     : vocleditor.edit.policies<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This policy is for editing a navigation expression<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class NavExpXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/**
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		Shell shell = ((NavExpEditPart) getHost()).getShell();
		Object newObjectType = request.getNewObjectType();
		Command createCommand = null;

		if (newObjectType == ClassifierRole.class) {
			CreateClassifierRoleSimpleCommand create =
				new CreateClassifierRoleSimpleCommand(shell);
			//			ClassifierRoleSimple simple = (ClassifierRoleSimple)request.getNewObject();
			ModelCreationFactory mfc =
				new ModelCreationFactory(ClassifierRole.class);
			ClassifierRole simple = (ClassifierRole) mfc.getNewObject();
			create.setClassifierRoleSimple(simple);
			create.setParent((NavExp) getHost().getModel());
			Rectangle rectangle = (Rectangle) getConstraintFor(request);
			create.setLocation(rectangle);
			create.setLabel("Create a SimpleClassifierRole");
			createCommand = create;
		} else if (newObjectType == ClassifierRoleSet.class) {
			CreateClassifierRoleSetCommand create =
				new CreateClassifierRoleSetCommand(shell);
			//						ClassitfierRoleSimple simple = (ClassifierRole)request.getNewObject();
			ModelCreationFactory mfc =
				new ModelCreationFactory(ClassifierRoleSet.class);
			ClassifierRoleSet simple = (ClassifierRoleSet) mfc.getNewObject();
			create.setClassifierRoleSet(simple);
			create.setParent((NavExp) getHost().getModel());
			Rectangle rectangle = (Rectangle) getConstraintFor(request);
			create.setLocation(rectangle);
			create.setLabel("Create a SetClassifierRole");
			createCommand = create;
		} else if (newObjectType == Condition.class) {
			CreateConditionCommand create = new CreateConditionCommand(shell);
			create.setParent(getHost().getModel());
			create.setCondition((Condition) request.getNewObject());
			createCommand = create;
		} 
		return createCommand;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	protected EditPolicy createChildEditPolicy(EditPart child) {
		return new NonResizableEditPolicy();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getMinimumSizeFor(org.eclipse.gef.GraphicalEditPart)
	 */
	protected Dimension getMinimumSizeFor(GraphicalEditPart child) {
		return child.getContentPane().getMinimumSize();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(
		EditPart child,
		Object constraint) {
		Command locationCommand = null;

		if (child instanceof ClassifierRoleSimpleEditPart
			|| child instanceof ClassifierRoleSetEditPart) {
			SetClassifierConstraintCommand command =
				new SetClassifierConstraintCommand();
			Object o = child.getModel();
			command.setPart((ClassifierNode) o);
			command.setLocation((Rectangle) constraint);
			locationCommand = command;
		} else if (child instanceof AssociationRoleEditPart) {

		}

		return locationCommand;
	}
}
