// File: PetriNetXYLayoutEditPolicy.java
// Adapted from PetriNetXYLayoutEditPolicy.java

package vocleditor.edit.policies;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.swt.widgets.Shell;

import vocleditor.command.ConstraintMoveCommand;
import vocleditor.command.CreateNewConstraintCommand;
import vocleditor.edit.parts.ConstraintEditPart;
import vocleditor.edit.parts.VOCLDiagrammEditPart;
import vocleditor.figures.ConstraintFigure;
import VOCLModel.Constraint;
import VOCLModel.VOCLDiagram;
/**
 * Class       : VOCLDiagrammXYLayoutEditPolicy<br/>
 * Package     : vocleditor.edit.policies<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This policy is for editing a VOCL diagram (adding constraints)<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class VOCLDiagrammXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/**
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createAddCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createAddCommand(EditPart child, Object constraint) {
		return null;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	protected Command getCreateCommand(CreateRequest request) {
		Shell shell = ((VOCLDiagrammEditPart) getHost()).getShell();
		Object newObjectType = request.getNewObjectType();
		Command createCommand = null;

		if (newObjectType == Constraint.class) {
			CreateNewConstraintCommand create =
				new CreateNewConstraintCommand(
					((VOCLDiagrammEditPart) getHost()).getShell());
			create.setConstraint((Constraint) request.getNewObject());
			create.setParent((VOCLDiagram) getHost().getModel());
			create.setLocation(request.getLocation());
			createCommand = create;
		}
		return createCommand;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	protected Command getDeleteDependantCommand(Request request) {
		return null;
	}

	/**
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	protected EditPolicy createChildEditPolicy(EditPart child) {
		return new ResizableEditPolicy();

	}

	/**
	 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getMinimumSizeFor(org.eclipse.gef.GraphicalEditPart)
	 */
	protected Dimension getMinimumSizeFor(GraphicalEditPart child) {
		return child.getContentPane().getMinimumSize();
	}

	/**
	 * @see org.eclipse.gef.editpolicies.ConstrainedLayoutEditPolicy#createChangeConstraintCommand(org.eclipse.gef.EditPart, java.lang.Object)
	 */
	protected Command createChangeConstraintCommand(
		EditPart child,
		Object constraint) {
		Command locationCommand = null;
		if (child instanceof ConstraintEditPart) {
			ConstraintFigure figure =
				((ConstraintEditPart) child).getConstraintFigure();

			ConstraintMoveCommand command = new ConstraintMoveCommand();
			command.setPart((Constraint) child.getModel());
			command.setFigure(figure);
			command.setLocation((Rectangle) constraint);
			locationCommand = command;
		}

		return locationCommand;
	}
}
