
package vocleditor.editor;

import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.editparts.ZoomListener;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.jface.util.ListenerList;

/**
 * A delegating ZoomManager.
 * 
 */
public class DelegatingZoomManager extends ZoomManager implements ZoomListener
{
    /** the current ZoomManager all work is delegated to */
    private ZoomManager currentZoomManager;

    /** listeners */
    private ListenerList zoomListeners = new ListenerList(3);

    /**
     * Creates a new DelegatingZoomManager instance.
     */
    public DelegatingZoomManager()
    {
        super((ScalableFigure) null, (Viewport) null);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomListener#zoomChanged(double)
     */
    public void zoomChanged(double zoom)
    {
        Object[] listeners = zoomListeners.getListeners();
        for (int i = 0; i < listeners.length; ++i)
        {
            ((ZoomListener) listeners[i]).zoomChanged(zoom);
        }
    }
    
    /**
     * @see org.eclipse.gef.editparts.ZoomManager#addZoomListener(org.eclipse.gef.editparts.ZoomListener)
     */
    public void addZoomListener(ZoomListener listener)
    {
        zoomListeners.add(listener);
    }
    
    /**
     * @see org.eclipse.gef.editparts.ZoomManager#removeZoomListener(org.eclipse.gef.editparts.ZoomListener)
     */
    public void removeZoomListener(ZoomListener listener)
    {
        zoomListeners.remove(listener);
    }

    /**
     * Sets the ZoomManager all work should be delegated to.
     * @param zoomManager
     */
    public void setCurrentZoomManager(ZoomManager zoomManager)
    {
        if (null != currentZoomManager)
            currentZoomManager.removeZoomListener(this);

        currentZoomManager = zoomManager;
        if(null != currentZoomManager)
        {
            currentZoomManager.addZoomListener(this);
            zoomChanged(currentZoomManager.getZoom());
        }
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#canZoomIn()
     */
    public boolean canZoomIn()
    {
        if(null == currentZoomManager)
            return false;
            
        return currentZoomManager.canZoomIn();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#canZoomOut()
     */
    public boolean canZoomOut()
    {
        if(null == currentZoomManager)
            return false;
            
        return currentZoomManager.canZoomOut();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getMaxZoom()
     */
    public double getMaxZoom()
    {
        if(null == currentZoomManager)
            return 1;
            
        return currentZoomManager.getMaxZoom();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getMinZoom()
     */
    public double getMinZoom()
    {
        if(null == currentZoomManager)
            return 1;
            
        return currentZoomManager.getMinZoom();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getNextZoomLevel()
     */
    public double getNextZoomLevel()
    {
        if(null == currentZoomManager)
            return 1;
            
        return currentZoomManager.getNextZoomLevel();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getPreviousZoomLevel()
     */
    public double getPreviousZoomLevel()
    {
        if(null == currentZoomManager)
            return 1;
            
        return currentZoomManager.getPreviousZoomLevel();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getScalableFigure()
     */
    public ScalableFigure getScalableFigure()
    {
        if(null == currentZoomManager)
           return null;
            
       return currentZoomManager.getScalableFigure();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getUIMultiplier()
     */
    public double getUIMultiplier()
    {
        if(null == currentZoomManager)
            return 1;
            
        return currentZoomManager.getUIMultiplier();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getViewport()
     */
    public Viewport getViewport()
    {
        if(null == currentZoomManager)
            return null;
            
        return currentZoomManager.getViewport();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getZoom()
     */
    public double getZoom()
    {
        if(null == currentZoomManager)
            return 1;
            
        return currentZoomManager.getZoom();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getZoomAsText()
     */
    public String getZoomAsText()
    {
        if(null == currentZoomManager)
            return " 100%";
            
        return currentZoomManager.getZoomAsText();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getZoomLevels()
     */
    public double[] getZoomLevels()
    {
        if(null == currentZoomManager)
            return new double[] {1};
            
        return currentZoomManager.getZoomLevels();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#getZoomLevelsAsText()
     */
    public String[] getZoomLevelsAsText()
    {
        if(null == currentZoomManager)
            return new String[] {" 100%"};
            
        return currentZoomManager.getZoomLevelsAsText();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setUIMultiplier(double)
     */
    public void setUIMultiplier(double multiplier)
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.setUIMultiplier(multiplier);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setViewLocation(org.eclipse.draw2d.geometry.Point)
     */
    public void setViewLocation(Point p)
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.setViewLocation(p);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setZoom(double)
     */
    public void setZoom(double zoom)
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.setZoom(zoom);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setZoomAnimationStyle(int)
     */
    public void setZoomAnimationStyle(int style)
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.setZoomAnimationStyle(style);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setZoomAsText(java.lang.String)
     */
    public void setZoomAsText(String zoomString)
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.setZoomAsText(zoomString);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#setZoomLevels(double[])
     */
    public void setZoomLevels(double[] zoomLevels)
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.setZoomLevels(zoomLevels);
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#zoomIn()
     */
    public void zoomIn()
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.zoomIn();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#zoomOut()
     */
    public void zoomOut()
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.zoomOut();
    }

    /**
     * @see org.eclipse.gef.editparts.ZoomManager#zoomTo(org.eclipse.draw2d.geometry.Rectangle)
     */
    public void zoomTo(Rectangle rect)
    {
        if(null == currentZoomManager)
            return ;
            
        currentZoomManager.zoomTo(rect);
    }

}
