package vocleditor.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseMotionListener;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.handles.HandleBounds;

import VOCLModel.Constraint;

/**
 * Class : BoolExpFigure <br/>Package : vocleditor.edit.parts <br/>Project :
 * VOCLEditor <br/>Created On : 22.05.2004 <br/>Description :<br/><br/>
 * 
 * @author Gruppe 1: Stefan Hnsgen, Andre Hernich, Maximilian Kern, Sebastian
 *         Russin <br>
 * 
 *  
 */
public class BoolExpFigure
	extends VOCLExpFigure
	implements HandleBounds, MouseMotionListener {

	//Layout for this Figure
	protected XYLayout layout;
	//Label for the operation (Or, XOR, IMPLIES)
	protected Label boolOperation;
	//the separation line
	protected PolylineConnection seperationLine_up;
	protected PolylineConnection seperationLine_down;
	//the figure on the upper / left side
	private Figure firstOp = null;
	//the figure on the bottom / right side
	private Figure secondOp = null;
	//help variables for layout computation
	int width = 60;
	int height = 60;
	int index = 0;

	/**
	 * Sets the figure on the left / top side.
	 * 
	 * @param fig
	 *            the figure
	 */
	public void setFirstOpFigure(Figure fig) {
		firstOp = fig;
	}

	/**
	 * Sets the figure on the right / bottom side.
	 * 
	 * @param fig
	 *            the figure
	 */
	public void setSecondOpFigure(Figure fig) {
		secondOp = fig;
	}

	/**
	 * Sets the index / position of this figure.
	 * 
	 * @param i
	 */
	public void setIndex(int i) {
		index = i;
	}

	public int getIndex() {
		return index;
	}

	/**
	 * Constructor. Set the Layout and other children like lines, figures of the
	 * BoolExpFigure.
	 */
	public BoolExpFigure() {
		layout = new XYLayout();
		setLayoutManager(layout);
		boolOperation = new Label();
		addMouseMotionListener(this);
		boolOperation.setOpaque(false);
		add(boolOperation);

		seperationLine_up = new PolylineConnection();
		add(seperationLine_up);
		seperationLine_down = new PolylineConnection();
		add(seperationLine_down);
		validate();
	}

	public void setBoolOperation(String op) {
		boolOperation.setText(op);
	}

	/**
	 * @see org.eclipse.draw2d.IFigure#add(org.eclipse.draw2d.IFigure,
	 *      java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) {
		//add the figure to the parent figure
		if (figure instanceof VOCLExpFigure) {
			int i = ((VOCLExpFigure) figure).getIndex();
			if (i == 1) {
				firstOp = (VOCLExpFigure) figure;
			} else if (i == 2) {
				secondOp = (VOCLExpFigure) figure;
			}
			super.add(figure, constraint, index);
		} else
			super.add(figure, constraint, index);
	}

	/**
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate() {
		
		//get the current width and height
		width = getSize().width;
		height = getSize().height;

		int labelwidth = boolOperation.getSize().width;
		int labelheight = boolOperation.getSize().height;

		if (isVertical) {
			//compute the layout if vertical
			seperationLine_up.setStart(new Point(width / 2, 0));
			seperationLine_up.setEnd(
				new Point(width / 2, (height / 2) - (labelheight / 2)));
			seperationLine_down.setStart(
				new Point(width / 2, (height / 2) + (labelheight / 2)));
			seperationLine_down.setEnd(new Point(width / 2, height));
			layout.setConstraint(
				firstOp,
				new Rectangle(1, 1, (width / 2)-2, height-2));
			layout.setConstraint(
				secondOp,
				new Rectangle((width / 2)+1, 1, (width / 2)-2, height-2));
		} else {
			//compute the layout if horizontal
			seperationLine_up.setStart(new Point(0, height / 2));
			seperationLine_up.setEnd(
				new Point((width / 2) - (labelwidth / 2), height / 2));
			seperationLine_down.setStart(
				new Point((width / 2) + (labelwidth / 2), height / 2));
			seperationLine_down.setEnd(new Point(width, height / 2));
			layout.setConstraint(
				firstOp,
				new Rectangle(1, 1, width-2, (height / 2)-2));
			layout.setConstraint(
				secondOp,
				new Rectangle(1, (height / 2)+1, width-2, (height / 2)-2));
		}
		//layout the boollabel
		layout.setConstraint(
			boolOperation,
			new Rectangle(
				width / 2 - (labelwidth / 2),
				height / 2 - (labelheight / 2),
				labelwidth,
				labelheight));
		super.validate();
	}

	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		if (layout.getConstraint(this) != null);
		return new Dimension(width, height);
	}

	/**
	 * @see org.eclipse.gef.handles.HandleBounds#getHandleBounds()
	 */
	public Rectangle getHandleBounds() {
		return getBounds().getCropped(new Insets(2, 0, 2, 0));
	}

	/**
	 * Sets the figure in vertical / horizontal manner.
	 * 
	 * @param b
	 */
	public void setVertical(boolean b) {
		isVertical = b;
	}

	/**
	 * @see org.eclipse.draw2d.MouseListener#mousePressed(org.eclipse.draw2d.MouseEvent)
	 */
	public void mousePressed(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseListener#mouseReleased(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseReleased(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseListener#mouseDoubleClicked(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseDoubleClicked(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseDragged(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseDragged(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseEntered(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseEntered(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseExited(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseExited(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseHover(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseHover(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseMoved(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseMoved(MouseEvent me) {
		p = me.getLocation();
	}

	//the coordinates for drawing the ClassifierRole
	Point p = new Point();

	/**
	 * Returns whether the figure should be place on the left or right side.
	 * 
	 * @return 1 - left side. 2 - right side.
	 */
	public int returnBereich() {
		if (isVertical) {
			if(index == 2){
				p.x -= width;
			}
			if (p.x < width / 2) {
				System.out.println(p.toString());
				return 1;
			} else {
				p.x -= width/2;
				System.out.println(p.toString());
				return 2;
			}
		} else {
			if (getParent() instanceof Constraint) {
				if ((p.y-30) < (height / 2)) {
//					System.out.println(p.toString());
					return 1;
				} else {
					p.y -= (height/2);
//					System.out.println(p.toString());
					return 2;
				}
			} else {
				if(index == 2 ){
					if(getParent() instanceof BoolExpFigure) {
						p.y = p.y - height;
					}else if (getParent() instanceof DecisionFigure){
						p.y = p.y - (height/3);
					}
							
				} 
				System.out.println("Point before calculation" + p.toString());
				if (p.y < (height / 2)) {
					System.out.println(p.toString() + " " +height + " 1");
					return 1;
				} else {
					System.out.println(p.toString() + " " +height+" 2");
					p.y -= (height/2);
					return 2;
				}
			}
		}
	}

	/**
	 * @see vocleditor.figures.VOCLExpFigure#getClassifierLocation()
	 */
	public Point getClassifierLocation() {
		return p;
	}

}
