package vocleditor.figures;

import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * 
 * Class       : ConditionFigure<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : <br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 *
 * Creates an ConditionFigure which is placed in Constraint or a NavigationExpression. 
 * The condition Text is set on a Label.
 */
public class ConditionFigure extends RectangleFigure {
	
	private XYLayout layout;
	//Label for the condition text
	private Label condition;
	
	public ConditionFigure(){
		layout = new XYLayout();
		setLayoutManager(layout);
		setOutline(false);
		condition = new Label("");
		add(condition);
	}
	
	/**
	 * Sets the condition text as a String
	 * @param cond
	 */
	public void setCondition(String cond){
		condition.setText(cond);
		//validate the condition figure
		validate();
	}
	
	/**
	 * Returns the condition as a String.
	 * @return a String representation
	 */
	public String getCondition(){
		return condition.getText();
	}
	
	/**
	 * @see org.eclipse.draw2d.Figure#validate()
	 */
	public void validate() {
		layout.setConstraint(condition,new Rectangle(5,5,-1,-1));
		super.validate();
	}
	
	/**
	 * @see org.eclipse.draw2d.Figure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width = 100;
		int height = condition.getSize().height;
		return new Dimension(width,height);
	}


}
