package vocleditor.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.RoundedRectangle;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;

/**
 * 
 * Class       : ConstraintFigure<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : <br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 *
 * Creates a ConstraintFigure which contains VOCLExpressionFigures.
 */
public class ConstraintFigure extends RoundedRectangle {

	protected XYLayout layout;
	protected Label contextLabel;
	//the line betweent the Context and the figures
	protected PolylineConnection contextLine;
	protected PolylineConnection conditionLine;
	protected RoundedRectangle constraint;

	ConditionFigure condFigure = new ConditionFigure();
	VOCLExpFigure voclFigure;

	//default width of the Constraintfigure
	private final static int CONSTRAINT_WIDTH = 503;

	public ConstraintFigure() {
		Font classFont = new Font(null, "Sans", 12, SWT.BOLD);
		layout = new XYLayout();
		setLayoutManager(layout);
		//		setOutline(true);
		conditionLine = new PolylineConnection();
		conditionLine.setLineStyle(Graphics.LINE_DASH);
		contextLine = new PolylineConnection();
		contextLabel = new Label();
		contextLabel.setFont(classFont);
		//contains contextname, -typ, constraintkind, constraintname
		add(contextLabel);
		
		contextLine.setStart(new Point(0, 29));
		contextLine.setEnd(new Point(CONSTRAINT_WIDTH - 1, 29));
		add(contextLine);
		setPreferredSize(new Dimension(CONSTRAINT_WIDTH, 300));
		setMinimumSize(new Dimension(CONSTRAINT_WIDTH, 300));
	}
	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * @see org.eclipse.draw2d.Figure#validate()
	 */
	public void validate() {
		int width = getPreferredSize().width;
		int height = getPreferredSize().height;
		if (!condFigure.getCondition().equals("")) {
			//if there is a condition
			layout.setConstraint(
				voclFigure,
				new Rectangle(1, 30, width - 3, height - 70));
			layout.setConstraint(
				condFigure,
				new Rectangle(1, height - 30, -1, 28));
			conditionLine.setStart(new Point(0, height - 35));
			conditionLine.setEnd(new Point(width, height - 35));
		} else {
			//if there is no condition
			conditionLine.setStart(new Point(0, 0));
			conditionLine.setEnd(new Point(0, 0));
			layout.setConstraint(condFigure, new Rectangle(0, 0, 0, 0));
			layout.setConstraint(contextLabel, new Rectangle(5, 5, -1, -1));
			contextLine.setStart(new Point(0, 29));
			contextLine.setEnd(new Point(width, 29));
			layout.setConstraint(
				voclFigure,
				new Rectangle(1, 30, width - 3, height - 40));
		}

		super.validate();
	}

	/**
	 * Sets the context text in the upper figure
	 * @param name
	 */
	public void setContext(String name) {
		contextLabel.setText(name);
		validate();
	}

	/**
	 * @see org.eclipse.draw2d.Figure#add(org.eclipse.draw2d.IFigure, java.lang.Object, int)
	 */

	public void add(IFigure figure, Object constraint, int index) {
		//Constraint for Voclexpression inserted in the Constraintfigure
		if (figure instanceof VOCLExpFigure) {
			voclFigure = (VOCLExpFigure) figure;
			super.add(figure, constraint, index);
		} else if (figure instanceof ConditionFigure) {
			condFigure = (ConditionFigure) figure;
			add(conditionLine);
			Rectangle condRec = new Rectangle(10, 335, -1, -1);
			super.add(figure, condRec, index);
		} else {
			super.add(figure, constraint, index);
		}
	}

}
