package vocleditor.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseMotionListener;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

import sun.security.action.GetPropertyAction;

/**
 * 
 * Class : DecisionFigure <br/>Package : vocleditor.edit.parts <br/>Project :
 * VOCLEditor <br/>Created On : 22.05.2004 <br/>Description :<br/><br/>
 * 
 * @author Gruppe 1: Stefan Hnsgen, Andre Hernich, Maximilian Kern, Sebastian
 *         Russin <br>
 * 
 * 
 * Creates a DecisionFigure which contains three figures. An if , then and
 * else-figure.
 */
public class DecisionFigure extends VOCLExpFigure implements
		MouseMotionListener {

	//the three different figures (of the type VOCLExpFigures)
	private Figure ifPart, thenPart, elsePart;

	//the labels
	private Label ifLabel, thenLabel, elseLabel;

	//the lines for seperating the figures
	private PolylineConnection ifFirstLine, ifSecondLine, ifThirdLine,
			thenElseLine;

	private XYLayout layout;

	private int width, height;

	//	private int index = 0;

	/**
	 * Constructor. Creates all elements like lines and figures.
	 */
	public DecisionFigure() {
		layout = new XYLayout();
		setLayoutManager(layout);

		ifLabel = new Label("if");
		thenLabel = new Label("then");
		elseLabel = new Label("else");
		ifFirstLine = new PolylineConnection();
		ifSecondLine = new PolylineConnection();
		ifThirdLine = new PolylineConnection();
		thenElseLine = new PolylineConnection();
		add(ifFirstLine);
		add(ifThirdLine);
		add(ifSecondLine);
		add(thenElseLine);
		add(ifLabel);
		add(thenLabel);
		add(elseLabel);
		//the mouselister for determining the positions of the if, then,
		// else-figures
		addMouseMotionListener(this);
	}

	/**
	 * @see org.eclipse.draw2d.Figure#add(org.eclipse.draw2d.IFigure,
	 *      java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) {
		//add figures to the if, then, else-figure
		if (figure instanceof VOCLExpFigure) {
			int i = ((VOCLExpFigure) figure).getIndex();
			if (i == 1) {
				ifPart = (VOCLExpFigure) figure;
			} else if (i == 2) {
				thenPart = (VOCLExpFigure) figure;
			} else if (i == 3) {
				elsePart = (VOCLExpFigure) figure;
			}
			super.add(figure, constraint, index);
		} else {
			super.add(figure, constraint, index);
		}
	}

	/**
	 * @see org.eclipse.draw2d.Figure#validate()
	 */
	public void validate() {
		width = getSize().width;
		height = getSize().height;

		//Labels are set correctly
		layout.setConstraint(ifLabel, new Rectangle(width / 2, 2, -1, -1));
		layout.setConstraint(thenLabel, new Rectangle((width) / 4,
				(height) / 3, -1, -1));
		layout.setConstraint(elseLabel, new Rectangle((3 * width) / 4,
				(height) / 3, -1, -1));

		//VoclExpFigures set correctly
		int thenLabelheight = thenLabel.getSize().height;
		layout.setConstraint(ifPart, new Rectangle(1, 2 + thenLabelheight,
				width - 2, height / 3 - thenLabelheight));

		layout.setConstraint(thenPart, new Rectangle(1, height / 3
				+ (thenLabelheight), (width / 2) - 1, (2 * height) / 3
				- thenLabelheight-1));

		layout.setConstraint(elsePart, new Rectangle((width / 2) + 1,
				(height / 3) + thenLabelheight, (width / 2) - 2,
				((2 * height) / 3) - thenLabelheight-1));

		//lines are set correctly
		ifFirstLine.setStart(new Point(0, height / 3));
		ifFirstLine.setEnd(new Point(width / 4, height / 3));
		ifSecondLine.setStart(new Point(
				(width / 4) + thenLabel.getSize().width, height / 3));
		ifSecondLine.setEnd(new Point((3 * width) / 4, height / 3));
		ifThirdLine.setStart(new Point((3 * width) / 4
				+ (elseLabel.getSize().width), height / 3));
		ifThirdLine.setEnd(new Point(width, height / 3));
		thenElseLine.setStart(new Point(width / 2, height / 3));
		thenElseLine.setEnd(new Point(width / 2, height));

		super.validate();
	}

	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseDragged(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseDragged(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseEntered(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseEntered(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseExited(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseExited(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseHover(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseHover(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseMoved(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseMoved(MouseEvent me) {
		p = me.getLocation();
	}

	Point p = new Point();

	/**
	 * @see vocleditor.figures.VOCLExpFigure#returnBereich()
	 */
	public int returnBereich() {

		if (getParent() instanceof ConstraintFigure) {
			if (p.y - 30 < (height / 3) + 5) {
				//positionate figure on the if-figure
				System.out.println("for if part: " + p.toString());
				return 1;
			} else {
				if (p.x < width / 2) {
					//positionate figure on the then-figure
					System.out.println("for then part: " + p.toString());
					return 2;
				} else {
					//positionate figure on the else-figure

					p.x -= width / 2;
					p.y -= height / 2;
					System.out.println("for else part: " + p.toString());
					return 3;
				}
			}
		} else {
			System.out
					.println("--------------------------------------------------"
							+ index + "-");
			if (index == 2 && !((VOCLExpFigure) getParent()).isVertical) {
				p.y = p.y - height;

//				System.out.println("aktuelle Position" + p.toString());
			} else if (index == 2 && ((VOCLExpFigure) getParent()).isVertical) {
				p.x = p.x - getParent().getSize().width / 2;
//				System.out.println("aktuelle Position" + p.toString());
			}
			if (p.y < (height / 3) + 5) {
				//positionate figure on the if-figure
								System.out.println("for if part: " + p.toString());
				return 1;
			} else {
				if (p.x < width / 2) {
					//positionate figure on the then-figure
										System.out.println("for then part: " + p.toString());
					if (((VOCLExpFigure) getParent()).isVertical)
						p.y -= height / 2;
					else
						p.x -= width / 2;
					return 2;
				} else {
					//positionate figure on the else-figure
					if (!((VOCLExpFigure) getParent()).isVertical)
						p.y -= height / 2;
					else
						p.x -= width / 2;
					//					p.y -= height / 2;
										System.out.println("for else part: " + p.toString());
					return 3;
				}
			}
		}

	}

	/**
	 * @see vocleditor.figures.VOCLExpFigure#getClassifierLocation()
	 */
	public Point getClassifierLocation() {
		return p;
	}

}