package vocleditor.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseMotionListener;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.handles.HandleBounds;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;

/**
 * 
 * Class       : LetFigure<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Creates a Let-In-ExpressionFigure.<br> 
 * On the left and right side can VOCLExpressionFigures be placed.<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class LetExpFigure
	extends VOCLExpFigure
	implements HandleBounds, MouseMotionListener {

	protected XYLayout layout;
	protected Label varLabel;
	protected Label letLabel;
	protected Label inLabel;
	protected PolylineConnection seperationLine_up;
	protected PolylineConnection seperationLine_down;
	protected PolylineConnection seperationLine_up2;
	protected PolylineConnection seperationLine_down2;
	private Figure letFigure = null;
	private Figure inFigure = null;
	int width = 60;
	int height = 60;

	public LetExpFigure() {
		layout = new XYLayout();
		setLayoutManager(layout);
		Font font = new Font(null, "Sans", 10, SWT.BOLD);
		varLabel = new Label();
		varLabel.setFont(font);

		letLabel = new Label(" Let ");
		letLabel.setOpaque(false);
		letLabel.setFont(font);

		inLabel = new Label(" In ");
		inLabel.setOpaque(false);
		inLabel.setFont(font);
		add(letLabel);
		add(inLabel);
		add(varLabel);

		seperationLine_up = new PolylineConnection();
		add(seperationLine_up);
		seperationLine_down = new PolylineConnection();
		add(seperationLine_down);
		seperationLine_up2 = new PolylineConnection();
		add(seperationLine_up2);
		seperationLine_down2 = new PolylineConnection();
		add(seperationLine_down2);

		addMouseMotionListener(this);

	}

	/**
	 * Sets the VariableDeclaration of this Figure.
	 * @param var
	 */
	public void setVariable(String var) {
		varLabel.setText(" " + var + " ");
	}

	/**
	 * @see org.eclipse.draw2d.IFigure#add(org.eclipse.draw2d.IFigure, java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) {
		if (figure instanceof VOCLExpFigure) {
			int i = ((VOCLExpFigure) figure).getIndex();
			if (i == 1) {
				letFigure = (VOCLExpFigure) figure;
			} else if (i == 2) {
				inFigure = (VOCLExpFigure) figure;
			}
			super.add(figure, constraint, index);
		} else {
			super.add(figure, constraint, index);
		}
	}

	/**
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate() {
		width = getSize().width;
		height = getSize().height;

		int inwidth = inLabel.getPreferredSize().width;
		int inheight = inLabel.getPreferredSize().height;
		int letwidth = letLabel.getPreferredSize().width;
		int letheight = letLabel.getPreferredSize().height;

		int varheight = varLabel.getPreferredSize().height;
		int varwidth = varLabel.getPreferredSize().width;
		if (isVertical) {
			seperationLine_up.setStart(new Point(width / 2, 0));
			seperationLine_up.setEnd(
				new Point(width / 2, (height / 2) - (inheight / 2)));
			seperationLine_down.setStart(
				new Point(width / 2, (height / 2) + (inheight / 2)));
			seperationLine_down.setEnd(new Point(width / 2, height));

			seperationLine_up2.setStart(new Point(letwidth / 2, 0));
			seperationLine_up2.setEnd(
				new Point(letwidth / 2, (height / 2) - (letheight / 2)));
			seperationLine_down2.setStart(
				new Point(letwidth / 2, (height / 2) + (letheight / 2)));
			seperationLine_down2.setEnd(new Point(letwidth / 2, height));

			layout.setConstraint(
				letFigure,
				new Rectangle(
					letwidth,
					varheight + 2,
					(4 * (width / 2)) / 5,
					((height - varheight)) - 10));
			layout.setConstraint(
				inFigure,
				new Rectangle((width / 2) + 9, 2, (width / 2) - 10, height - 5));
			layout.setConstraint(
				varLabel,
				new Rectangle(letwidth, 3, varwidth, varheight));

			layout.setConstraint(
				letLabel,
				new Rectangle(
					0,
					height / 2 - letheight / 2,
					letwidth,
					letheight));
		} else {
			seperationLine_up.setStart(new Point(0, height / 2));
			seperationLine_up.setEnd(
				new Point((width / 2) - (inwidth / 2), height / 2));
			seperationLine_down.setStart(
				new Point((width / 2) + (inwidth / 2), height / 2));
			seperationLine_down.setEnd(new Point(width, height / 2));

			seperationLine_up2.setStart(new Point(0, letheight / 2));
			seperationLine_up2.setEnd(
				new Point((width / 2) - letwidth / 2, letheight / 2));
			seperationLine_down2.setStart(
				new Point((width / 2) + letwidth / 2, letheight / 2));
			seperationLine_down2.setEnd(new Point(width, letheight / 2));

			layout.setConstraint(
				letLabel,
				new Rectangle(
					width / 2 - letwidth / 2,
					0,
					letwidth,
					letheight));
			layout.setConstraint(
				varLabel,
				new Rectangle(0, letheight, varwidth, varheight));

			layout.setConstraint(
				letFigure,
				new Rectangle(0, varheight, width, height / 2 - varheight));
			layout.setConstraint(
				inFigure,
				new Rectangle(0, height / 2, width, height / 2));
		}

		layout.setConstraint(
			inLabel,
			new Rectangle(
				width / 2 - (inwidth / 2),
				height / 2 - (inheight / 2),
				inwidth,
				inheight));
		//		layout.setConstraint(fig, new Rectangle(0, 0, width / 2, height));

		super.validate();
	}

	/**
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * @see org.eclipse.gef.handles.HandleBounds#getHandleBounds()
	 */
	public Rectangle getHandleBounds() {
		return getBounds().getCropped(new Insets(2, 0, 2, 0));
	}

	/**
	 * Set whether this figure has a vertical / horizontal alignment.
	 * @param b
	 */
	public void setVertical(boolean b) {
		isVertical = b;
	}

	Point location = new Point();
	/**
	 * @see vocleditor.figures.VOCLExpFigure#returnBereich()
	 */
	public int returnBereich() {
		if (isVertical) {
			if (location.y > height) {
				location.y -= height;
			}
			if (location.x < width / 2) {
				//set the figure in the let-part
				return 1;
			} else {
				location.x -= width / 2;
				System.out.println(location.toString());
				//set the figure in the in-part
				return 2;

			}
		} else {
			if (location.y < height / 2) {
				//set the figure in the left-part
				return 1;
			} else {
				//set the figure in the right-part
				return 2;
			}
		}
	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseDragged(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseDragged(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseEntered(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseEntered(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseExited(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseExited(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseHover(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseHover(MouseEvent me) {

	}

	/**
	 * @see org.eclipse.draw2d.MouseMotionListener#mouseMoved(org.eclipse.draw2d.MouseEvent)
	 */
	public void mouseMoved(MouseEvent me) {
		location = me.getLocation();
	}

	/**
	 * @see vocleditor.figures.VOCLExpFigure#getClassifierLocation()
	 */
	public Point getClassifierLocation() {
		return location;
	}

}
