package vocleditor.figures;

import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.draw2d.MouseMotionListener;
import org.eclipse.draw2d.Polyline;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

import vocleditor.model.IAnchorFigure;
import vocleditor.model.NavExpConstants;
import vocleditor.model.NavExpEditorAnchor;

/**
 * @author Stefan
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
/**
 * 
 * Class       : NavExpContainerFigure<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Creates a ClassifierRoleSetFigure<br>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class SetClassifierRoleFigure
	extends RectangleFigure
	implements IAnchorFigure {

	private final static double SHADOW_PERCENTAGE = 0.1;
	private RectangleFigure frontRectangle;
	private RectangleFigure backRectangle;
	private Label varLabel;
	private Label opVarLabel;
	private Label opSignLabel;
	private Label attrLabel;
	private Dimension figureDimension;

	public static Color classColor = new Color(null, 255, 255, 206);
	//Anchors
	protected Vector anchorsList;
	private RectangleFigure operationBox;
	protected RectangleFigure box;
	private int boxSize = 20;
	private Point mousePoint;
	private int operationType;
	private String opVarName;
	protected Polyline line;
	private Rectangle opBoxConstraint;
	private PolylineConnection nullLine;
	private PolylineConnection notEqualLine;
	private PolylineConnection underTitle;
	protected Vector incomingAnchorsList;
	protected Vector outgoingAnchorsList;

	/**
	 * Constructor.
	 */
	public SetClassifierRoleFigure() {
		Font classFont = new Font(null, "Sans", 12, SWT.BOLD);
		setOutline(false);

		setLayoutManager(new XYLayout());
		backRectangle = new RectangleFigure();

		varLabel = new Label("");
		varLabel.setFont(classFont);
		varLabel.setTextAlignment(PositionConstants.CENTER);
		attrLabel = new Label();

		frontRectangle = new RectangleFigure();
		frontRectangle.setLayoutManager(new XYLayout());
		frontRectangle.add(varLabel);

		frontRectangle.add(attrLabel);
		frontRectangle.setOutline(true);
		frontRectangle.setOpaque(true);
		frontRectangle.setBackgroundColor(classColor);

		underTitle = new PolylineConnection();
		frontRectangle.add(underTitle);

		setOpaque(true);
		add(createOperationBox());
		add(backRectangle);
		add(frontRectangle);

		NavExpEditorAnchor.attachAndRefreshAnchors(this);

		mousePoint = new Point();
		this.addMouseListener(new MouseListener.Stub() {

			public void mousePressed(MouseEvent me) {
				mousePoint.x = me.x;
				mousePoint.y = me.y;
			}

		});

		this.addMouseMotionListener(new MouseMotionListener.Stub() {

			public void mouseMoved(MouseEvent me) {
				mousePoint.x = me.x;
				mousePoint.y = me.y;
			}
		});

	}

	/**
	 * Creates the visual representation of the operation on the right side.
	 * @return the figure including the operation
	 */
	public RectangleFigure createOperationBox() {
		operationBox = new RectangleFigure();
		operationBox.setLayoutManager(new XYLayout());
		operationBox.setLineStyle(3);
		opVarLabel = new Label();
		opSignLabel = new Label();
		operationBox.add(opVarLabel);
		operationBox.add(opSignLabel);
		operationBox.add(nullLine = new PolylineConnection());
		operationBox.add(notEqualLine = new PolylineConnection());

		return operationBox;
	}

	/**
	 * Sets the operation name.
	 * @param name operation name.
	 */
	public void setOpVarName(String name) {
		opVarLabel.setText(name);
	}

	/**
	 * Sets the attributes. They are represented only as one String.
	 * @param name the attributes as String.
	 */
	public void setAttrName(String name) {
		attrLabel.setText(name);
	}

	/**
	 * Help method for computing the layout of the operationBox.
	 * @param opBoxPos
	 * @param height
	 */
	private void refreshOperationBox(Point opBoxPos, int height) {
		if (opVarLabel.getText() == null || opVarLabel.getText() == "") {
			opVarLabel.setText("SCHOENECLIPSE");
		}
		switch (operationType) {
			case 0 :
			case 1 :
				opVarLabel.setText("0");
				opSignLabel.setText("=");
				break;
			case 2 :
				opSignLabel.setText("#");
				break;
			case 3 :
				opSignLabel.setText("SUM");
		}
		//Check right size for operationBoxLabels ...
		int ySize = 3;
		Font opFont = null;
		while (opVarLabel.getPreferredSize().height + 6 <= height) {
			ySize++;
			opFont = new Font(null, "Sans", ySize, SWT.BOLD);
			opVarLabel.setFont(opFont);
		}
		opSignLabel.setFont(opFont);
		Rectangle opSignConstraint =
			new Rectangle(
				3,
				0,
				opSignLabel.getPreferredSize().width,
				opSignLabel.getPreferredSize().height + 6);
		Rectangle opVarConstraint =
			new Rectangle(
				3 + opSignLabel.getPreferredSize().width,
				0,
				opVarLabel.getPreferredSize().width,
				opVarLabel.getPreferredSize().height + 6);

		operationBox.getLayoutManager().setConstraint(
			opVarLabel,
			opVarConstraint);
		operationBox.getLayoutManager().setConstraint(
			opSignLabel,
			opSignConstraint);

		//Make lines through "0" and "=" ...					
		switch (operationType) {
			case 1 :
				Point start =
					new Point(opSignConstraint.x, opSignConstraint.height - 6);
				Point end =
					new Point(
						opSignConstraint.x + opSignConstraint.width - 3,
						opSignConstraint.y + 6);

				start.translate(opBoxPos);
				end.translate(opBoxPos);
				notEqualLine.setStart(start);
				notEqualLine.setEnd(end);
				break;
			case 0 :
				start =
					new Point(opVarConstraint.x, opVarConstraint.height - 3);
				end =
					new Point(
						opVarConstraint.x + opVarConstraint.width,
						opVarConstraint.y + 3);

				//operationBox.getLayoutManager().setConstraint(nullLine, new Rectangle(start.x,end.y, end.x - start.x, start.y - end.y));
				start.translate(opBoxPos);
				end.translate(opBoxPos);
				nullLine.setStart(start);
				nullLine.setEnd(end);
				break;
			case 2 :
			case 3 :
				nullLine.setStart(new Point(0, 0));
				nullLine.setEnd(new Point(0, 0));
				notEqualLine.setStart(new Point(0, 0));
				notEqualLine.setEnd(new Point(0, 0));
				break;

		}

	}

	/**
	 * Sets the Type for the ClassifierRoleSet.
	 * @param name the type for the ClassifierRoleSet
	 */
	public void setClass(String name) {
		varLabel.setText(": " + name);
		super.validate();
	}

	/**
	 * Sets the name for a ClassifierRoleSet.
	 * @param name name of the ClassifierRoleSet
	 */
	public void setVariable(String name) {
		String temp = varLabel.getText();
		temp = name + " " + temp;
		varLabel.setText(temp);
		super.validate();
	}

	/**
	 * Set the operation as a String.
	 * @param operType
	 */
	public void setOperationType(String operType) {
		for (int x = 0; x < NavExpConstants.CLASSIFIER_SET_TYPES.length; x++) {
			if (NavExpConstants.CLASSIFIER_SET_TYPES[x].equals(operType)) {
				operationType = x;
			}
		}
	}

	/**
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate() {
		// Location of the varLabel
		int attrwidth = attrLabel.getPreferredSize().width + 10;
		int attrheight = attrLabel.getPreferredSize().height + 10;
		if (attrLabel.getText().equals("") || attrLabel.getText() == null) {
			attrwidth = 0;
			attrheight = 0;
		}

		Rectangle varConstraint =
			new Rectangle(
				0,
				0,
				varLabel.getPreferredSize().width + 10,
				varLabel.getPreferredSize().height + 10);

		frontRectangle.getLayoutManager().setConstraint(
			varLabel,
			varConstraint);

		// Location of the attrLabel
		Rectangle attrConstraint =
			new Rectangle(
				3,
				varLabel.getPreferredSize().height + 10,
				attrwidth,
				attrheight);

		if (attrLabel.getText() == null || attrLabel.getText().equals("")) {
			underTitle.setVisible(false);
		} else {
			underTitle.setVisible(true);
			underTitle.setStart(new Point(0, attrConstraint.getLocation().y));
			underTitle.setEnd(
				new Point(
					frontRectangle.getSize().width,
					attrConstraint.getLocation().y));

		}
		frontRectangle.getLayoutManager().setConstraint(
			attrLabel,
			attrConstraint);

		Rectangle frontConstraint =
			new Rectangle(
				0,
				getRelativeShadow().y,
				Math.max(varConstraint.width, attrConstraint.width),
				varConstraint.height + attrConstraint.height);

		//frontRectangle
		getLayoutManager().setConstraint(frontRectangle, frontConstraint);

		//BackRectangle
		Rectangle backConstraint = new Rectangle(frontConstraint);
		backConstraint.setLocation(getRelativeShadow().x, 0);
		getLayoutManager().setConstraint(backRectangle, backConstraint);

		//Operationbox	
		int backheight = varLabel.getPreferredSize().height + 10 + attrheight;
		int opBoxHeight = (int) (backheight * 0.2);
		if (opBoxHeight < 20) {
			opBoxHeight = 20;
		}
		Point opBoxPos =
			new Point(
				frontConstraint.width + getRelativeShadow().x - 1,
				backConstraint.height - opBoxHeight);
		refreshOperationBox(opBoxPos, opBoxHeight);

		opBoxConstraint =
			new Rectangle(
				0,
				0,
				6
					+ opSignLabel.getPreferredSize().width
					+ opVarLabel.getPreferredSize().width,
				opBoxHeight);
		opBoxConstraint.setLocation(opBoxPos);
		getLayoutManager().setConstraint(operationBox, opBoxConstraint);

		//getLayoutManager().setConstraint(this, new Rectangle(0,0,getSize().width,getSize().height));

		NavExpEditorAnchor.attachAndRefreshAnchors(this);

		super.validate();
	}

	/**
		 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
		 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * @see org.eclipse.draw2d.Figure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		return super.getMinimumSize(wHint, hHint);
	}

	/**
	 * @see org.eclipse.draw2d.Figure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width = 0;
		int height = 0;
		int attrwidth = attrLabel.getPreferredSize().width + 10;
		int attrheight = attrLabel.getPreferredSize().height + 10;
		if (attrLabel.getText().equals("") || attrLabel.getText() == null) {
			attrwidth = 0;
			attrheight = 0;
		}
		figureDimension =
			new Dimension(
				Math.max(varLabel.getPreferredSize().width + 10, attrwidth)
					+ getRelativeShadow().x
					+ opVarLabel.getPreferredSize().width
					+ 6
					+ opSignLabel.getPreferredSize().width
					- 1,
				varLabel.getPreferredSize().height
					+ 10
					+ attrheight
					+ getRelativeShadow().y);
		return figureDimension;
	}

	private Point getRelativeShadow() {
		int attrwidth = attrLabel.getPreferredSize().width + 10;
		int attrheight = attrLabel.getPreferredSize().height + 10;
		if (attrLabel.getText().equals("") || attrLabel.getText() == null) {
			attrwidth = 0;
			attrheight = 0;
		}
		int x =
			(int) (SHADOW_PERCENTAGE
				* (attrwidth + varLabel.getPreferredSize().width + 10));
		int y =
			(int) (SHADOW_PERCENTAGE
				* (attrheight + varLabel.getPreferredSize().height + 10));
		x = (x < 5) ? 5 : x;
		y = (y < 5) ? 5 : y;
		return new Point(x, y);
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#setAnchorList(java.util.Vector)
	 */
	public void setAnchorList(Vector anchorsList) {
		this.anchorsList = anchorsList;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getAnchorList()
	 */
	public Vector getAnchorList() {
		return anchorsList;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getFigure()
	 */
	public IFigure getFigure() {
		return frontRectangle;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getMousePoint()
	 */
	public Point getMousePoint() {
		return mousePoint;
	}

	/**
	 * @see NavExpEditor.model.IAnchorFigure#setMousePoint(org.eclipse.draw2d.geometry.Point)
	 */
	public void setMousePoint(Point mousePoint) {

	}

	/**
		 * Returns the anchor for incoming connections.
		 */
	public NavExpEditorAnchor getSourceConnectionAnchor() {
		return NavExpEditorAnchor.getClosestConnectionAnchor(
			this,
			getSourceAnchorList());
	}
	/**
		 * @see NavExpEditor.figures.IAnchorFigure#getAnchorList()
		 */
	public Vector getTargetAnchorList() {
		return incomingAnchorsList;
	}

	/**
		 * Returns the anchor for incoming connections.
		 */
	public NavExpEditorAnchor getTargetConnectionAnchor() {
		return NavExpEditorAnchor.getClosestConnectionAnchor(
			this,
			getTargetAnchorList());
	}

	/**
		 * @see NavExpEditor.figures.IAnchorFigure#getAnchorList()
		 */
	public Vector getSourceAnchorList() {
		return outgoingAnchorsList;
	}

	/**
		 * @see NavExpEditor.figures.IAnchorFigure#setAnchorList(java.util.Vector)
		 */
	public void setTargetAnchorList(Vector anchorsList) {
		this.incomingAnchorsList = anchorsList;
	}

	/**
	* @see NavExpEditor.figures.IAnchorFigure#setAnchorList(java.util.Vector)
	*/
	public void setSourceAnchorList(Vector anchorsList) {
		this.outgoingAnchorsList = anchorsList;
	}

}
