package vocleditor.figures;

import java.util.Vector;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.draw2d.MouseMotionListener;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

import vocleditor.model.IAnchorFigure;
import vocleditor.model.NavExpEditorAnchor;

/**
 * 
 * Class       : SimpleClassifierRoleFigure<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : Creates a ClassifierRoleFigure.<br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 */
public class SimpleClassifierRoleFigure
	extends RectangleFigure
	implements IAnchorFigure {

	private Label varLabel;
	private Label attrLabel;
	public static Color classColor = new Color(null, 255, 255, 206);
	//Anchors
	protected Vector anchorsList;
	private Point mousePoint;
	private static int LINE_LENGTH = 60;
	protected RectangleFigure figureBox;
	protected RectangleFigure variableBox;
	protected RectangleFigure attributeBox;
	private boolean hasMethod = false;
	protected XYLayout layout;
	protected XYLayout figureBoxLayout;
	protected XYLayout varLayout;
	protected XYLayout attrLayout;
	protected PolylineConnection methodLine;
	protected Label methodLabel;
	protected Vector incomingAnchorsList;
	protected Vector outgoingAnchorsList;

	public SimpleClassifierRoleFigure() {
		Font classFont = new Font(null, "Sans", 12, SWT.BOLD);
		layout = new XYLayout();
		setLayoutManager(layout);
		setOutline(false);
		setOpaque(true);
		figureBox = new RectangleFigure();
		figureBoxLayout = new XYLayout();
		figureBox.setLayoutManager(figureBoxLayout);
		variableBox = new RectangleFigure();
		varLayout = new XYLayout();
		variableBox.setLayoutManager(varLayout);
		attributeBox = new RectangleFigure();
		attributeBox.setBackgroundColor(classColor);
		variableBox.setBackgroundColor(classColor);

		attrLayout = new XYLayout();
		attributeBox.setLayoutManager(attrLayout);

		attrLabel = new Label();
		varLabel = new Label();

		varLabel.setFont(classFont);
		variableBox.add(varLabel);
		attributeBox.add(attrLabel);
		figureBox.add(variableBox);
		figureBox.add(attributeBox);
		methodLabel = new Label();

		add(methodLabel);
		add(figureBox);

		NavExpEditorAnchor.attachAndRefreshAnchors(this);
		mousePoint = new Point();
		this.addMouseListener(new MouseListener.Stub() {

			public void mousePressed(MouseEvent me) {
				mousePoint.x = me.x;
				mousePoint.y = me.y;
			}

			public void mouseDoubleClicked(MouseEvent e) {
				System.out.println(e.getSource().toString());
			}

		});

		this.addMouseMotionListener(new MouseMotionListener.Stub() {

			public void mouseMoved(MouseEvent me) {
				mousePoint.x = me.x;
				mousePoint.y = me.y;
			}
		});

		//data for the method
		methodLine = new PolylineConnection();
		methodLine.setStart(new Point(0, 0));
		methodLine.setEnd(new Point(0, 0));
		PolygonDecoration pfeilSpitze = new PolygonDecoration();
		PointList points = new PointList();
		points.addPoint(-2, -2);
		points.addPoint(0, 0);
		points.addPoint(-2, 2);
		pfeilSpitze.setTemplate(points);
		methodLine.setTargetDecoration(pfeilSpitze);

	}

	/**
	 * Sets the type of the ClassifierRole.
	 * @param name type of the ClassifierRole
	 */
	public void setClass(String name) {
		varLabel.setText(": " + name);
//		validate();
	}

	/**
	 * Sets the name of the ClassifierRole
	 * @param name name of the ClassifierRole
	 */
	public void setVariable(String name) {
		String temp = varLabel.getText();
		temp = name + " " + temp;
		varLabel.setText(temp);
		validate();
	}

	/**
	 * Sets the attributes as one String.
	 * @param name attributes as one String
	 */
	public void setAttribute(String name) {
		attrLabel.setText(name);
		validate();
	}

	/**
	 * Set the method of a ClassifierRole.
	 * @param name the whole method-String that is displayed
	 */
	public void setMethod(String name) {
		methodLabel.setText(name);
		add(methodLine);
		hasMethod = true;
		validate();
	}


	/**
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate() {

		int width = getSize().width;
		int height = getSize().height;

		// Location of the whole figure
		Rectangle boxRect = new Rectangle(0, 0, -1, -1);

		// Location of the VarLabel in the variableRec
		Rectangle varConstraint = new Rectangle(5, 5, -1, -1);
//		varConstraint.setSize(varLabel.getSize().width,varLabel.getSize().height);
		varLayout.setConstraint(varLabel, varConstraint);

		Rectangle attrConstraint = new Rectangle(0, 0, -1, -1);
		attrConstraint.x = varConstraint.x;
		attrConstraint.y = varConstraint.y;
		attrLayout.setConstraint(attrLabel, attrConstraint);

		//		Location of the variableRec
		Rectangle varRectangle = new Rectangle(0, 0, 0, 0);
		varRectangle.height = varLabel.getSize().height + 10;

		Rectangle attrRectangle = new Rectangle(0, 0, 0, 0);
		attrRectangle.x = varRectangle.x;
		attrRectangle.y = varRectangle.y + varRectangle.height;
		attrRectangle.height = attrLabel.getSize().height + 10;

		int breite = 0;
		if (attrLabel.getText().equals("")) {
			breite = varLabel.getSize().width + 10;
			varRectangle.width = varLabel.getSize().width + 10;
			attrRectangle.width = varLabel.getSize().width + 10;
		} else {

			if (varLabel.getSize().width > attrLabel.getSize().width) {
				breite = varLabel.getSize().width + 10;
				varRectangle.width = varLabel.getSize().width + 10;
				attrRectangle.width = varLabel.getSize().width + 10;
			} else {
				breite = attrLabel.getSize().width + 10;
				varRectangle.width = attrLabel.getSize().width + 10;
				attrRectangle.width = attrLabel.getSize().width + 10;
			}

		}
		Rectangle figureRectangle = new Rectangle(0, 0, 0, 0);
		figureRectangle.width = breite;
		if (attrLabel.getText().equals("")) {
			figureRectangle.height = varRectangle.height;
			figureRectangle.y = height - varRectangle.height;
		} else {
			figureRectangle.height = attrRectangle.height + varRectangle.height;
			figureRectangle.y =
				height - attrRectangle.height - varRectangle.height;
		}

		figureBoxLayout.setConstraint(variableBox, varRectangle);
		figureBoxLayout.setConstraint(attributeBox, attrRectangle);

		layout.setConstraint(figureBox, figureRectangle);

		//if there exists a method
		if (hasMethod) {
			layout.setConstraint(
				methodLabel,
				new Rectangle(
					(varRectangle.width / 2) + 4,
					LINE_LENGTH / 2,
					-1,
					-1));

			Rectangle lineConstraint =
				new Rectangle(
					figureRectangle.width / 2,
					0,
					1,
					height - figureRectangle.height);
			methodLine.setStart(new Point(figureRectangle.width / 2, 0));
			methodLine.setEnd(
				new Point(figureRectangle.width / 2, LINE_LENGTH));
			layout.setConstraint(methodLine, lineConstraint);
		}
//		boxRect.width = width;
//		boxRect.height = height;
//
//		layout.setConstraint(this, boxRect);
		NavExpEditorAnchor.attachAndRefreshAnchors(this);
		super.validate();
	}

	/**
		 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
		 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	/**
	 * @see org.eclipse.draw2d.Figure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width = 0;
		int height = 0;
		int varLabelwidth = varLabel.getSize().width;
		int attrLabelwidth = attrLabel.getSize().width;
		int methodLabelwidth;
		if (hasMethod) {
			methodLabelwidth = methodLabel.getSize().width;
			if (attrLabel.getText().equals("")) {
				//				kein Attribut vorhanden
				height = varLabel.getSize().height + LINE_LENGTH + 10;
				if (methodLabelwidth < (varLabelwidth / 2)) {
					//MethodenLabel kleiner als varLabel
					width = varLabelwidth + 10;
				} else {
					width = varLabelwidth / 2 + methodLabelwidth + 10;
				}
			} else {
				//Attribut vorhanden
				height =
					varLabel.getSize().height
						+ attrLabel.getSize().height
						+ LINE_LENGTH
						+ 20;
				if (varLabelwidth < attrLabelwidth) {
					//Var ist kleiner als Attribut
					if (methodLabelwidth < (attrLabelwidth / 2)) {
						//MethodenLabel kleiner als varLabel
						width = attrLabelwidth + 10;
					} else {
						width = attrLabelwidth / 2 + methodLabelwidth + 10;
					}
				} else {
					if (methodLabelwidth < (varLabelwidth / 2)) {
						//MethodenLabel kleiner als varLabel
						width = varLabelwidth + 10;
					} else {
						width = varLabelwidth / 2 + methodLabelwidth + 10;
					}
				}
			}
		} else {

			//kein Attribut vorhanden
			if (attrLabel.getText().equals("")) {
				width = varLabel.getSize().width + 10;
				height = varLabel.getSize().height + 10;
			} else {
				//VariableDeclaration ist breiter als alle Attribute
				if (varLabel.getSize().width > attrLabel.getSize().width) {
					width = varLabel.getSize().width + 10;
					height =
						varLabel.getSize().height
							+ attrLabel.getSize().height
							+ 20;
				} else {
					width = attrLabel.getSize().width + 10;
					height =
						varLabel.getSize().height
							+ attrLabel.getSize().height
							+ 20;
				}
			}
		}
		return new Dimension(width, height);
	}
	/**
	 * @see NavExpEditor.figures.IAnchorFigure#setAnchorList(java.util.Vector)
	 */
	public void setAnchorList(Vector anchorsList) {
		this.anchorsList = anchorsList;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getAnchorList()
	 */
	public Vector getAnchorList() {
		return anchorsList;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getFigure()
	 */
	public IFigure getFigure() {
		return figureBox;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getMousePoint()
	 */
	public Point getMousePoint() {
		return mousePoint;
	}

	/**
	 * @see NavExpEditor.model.IAnchorFigure#setMousePoint(org.eclipse.draw2d.geometry.Point)
	 */
	public void setMousePoint(Point mousePoint) {

	}

	/**
		 * @see NavExpEditor.figures.IAnchorFigure#setAnchorList(java.util.Vector)
		 */
	public void setSourceAnchorList(Vector anchorsList) {
		this.outgoingAnchorsList = anchorsList;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getAnchorList()
	 */
	public Vector getSourceAnchorList() {
		return outgoingAnchorsList;
	}

	/**
		 * @see NavExpEditor.figures.IAnchorFigure#setAnchorList(java.util.Vector)
		 */
	public void setTargetAnchorList(Vector anchorsList) {
		this.incomingAnchorsList = anchorsList;
	}

	/**
	 * @see NavExpEditor.figures.IAnchorFigure#getAnchorList()
	 */
	public Vector getTargetAnchorList() {
		return incomingAnchorsList;
	}

	/**
		 * Returns the anchor for incoming connections.
		 */
	public NavExpEditorAnchor getSourceConnectionAnchor() {

		return NavExpEditorAnchor.getClosestConnectionAnchor(
			this,
			getSourceAnchorList());
	}

	/**
		 * Returns the anchor for incoming connections.
		 */
	public NavExpEditorAnchor getTargetConnectionAnchor() {
		return NavExpEditorAnchor.getClosestConnectionAnchor(
			this,
			getTargetAnchorList());
	}

}
