package vocleditor.model;

import java.util.Map;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.gef.requests.CreationFactory;

import VOCLModel.AssociationEndRoleSource;
import VOCLModel.AssociationEndRoleTarget;
import VOCLModel.AssociationRole;
import VOCLModel.Attribute;
import VOCLModel.BoolExp;
import VOCLModel.ClassifierRole;
import VOCLModel.ClassifierRoleSet;
import VOCLModel.Condition;
import VOCLModel.Constraint;
import VOCLModel.IfExp;
import VOCLModel.LetExp;
import VOCLModel.Method;
import VOCLModel.NavExp;
import VOCLModel.VOCLModelFactory;
import VOCLModel.VOCLModelPackage;
import VOCLModel.VariableDeclaration;



/**
 * Class       : ModelCreationFactory<br/>
 * Package     : vocleditor.model<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This class represents a factory for model elements.<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class ModelCreationFactory implements CreationFactory {

	private Class targetClass;

	public ModelCreationFactory(Class targetClass) {
		this.targetClass = targetClass;
	}
	/**
	 * @see org.eclipse.gef.requests.CreationFactory#getNewObject()
	 */
	public Object getNewObject() {
		Map registry = EPackage.Registry.INSTANCE;
		String workflowURI = VOCLModelPackage.eNS_URI;
		VOCLModelPackage navExpPackage =
			(VOCLModelPackage) registry.get(workflowURI);
		VOCLModelFactory factory = navExpPackage.getVOCLModelFactory();

		Object result = null;

		if (targetClass.equals(ClassifierRole.class)) {
			result = factory.createClassifierRole();
		} else if (targetClass.equals(ClassifierRoleSet.class)) {
			result = factory.createClassifierRoleSet();			
		} else if (targetClass.equals(AssociationRole.class)) {
			result = factory.createAssociationRole();
		} else if (targetClass.equals(AssociationEndRoleSource.class)) {
			result = factory.createAssociationEndRoleSource();
		} else if (targetClass.equals(AssociationEndRoleTarget.class)) {
			result = factory.createAssociationEndRoleTarget();
		} else if (targetClass.equals(VariableDeclaration.class)) {
			result = factory.createVariableDeclaration();
		} else if (targetClass.equals(Attribute.class)) {
			result = factory.createAttribute();
		} else if (targetClass.equals(Constraint.class)){
			result = factory.createConstraint();
		} else if (targetClass.equals(Method.class)){
			result = factory.createMethod();
		} else if (targetClass.equals(BoolExp.class)){
			result = factory.createBoolExp();
		} else if(targetClass.equals(NavExp.class)){
			result = factory.createNavExp();
		} else if(targetClass.equals(Condition.class)){
			result = factory.createCondition();
		} else if(targetClass.equals(IfExp.class)){
			result = factory.createIfExp();
		} else if(targetClass.equals(LetExp.class)){
			result = factory.createLetExp();
		}
		
		return result;

	}


	/**
	 * @see org.eclipse.gef.requests.CreationFactory#getObjectType()
	 */
	public Object getObjectType() {
		return targetClass;
	}

}
