/* 
 * $Id: ElementStatus.java,v 1.2 2004/11/09 09:49:37 haensgen Exp $
 */
 
package VoclToOclConverter;

import org.eclipse.core.runtime.Status;

import org.eclipse.emf.ecore.EObject;

/**
 * Contains information about a model element in an EMF model instance.
 */
public class ElementStatus extends Status {
	
	// The root of the model which contains the element
	private EObject root;
	
	// The element itself
	private EObject element;
	
	/**
	 * Creates a new ElementStatus instance for the given code, message, model
	 * root and model element. The model root is the root element of the EMF 
	 * model, e.g. a VOCLDiagram instance. The element is a direct or indirect
	 * child of the root. 
	 * 
	 * @param code
	 *     A code which describes the kind of status. See 
	 *     <tt>IElementStatusConstants</tt> for such codes.
	 * @param message
	 *     A human readable and localized message which describes the status.
	 * @param root
	 *     The root element of the EMF model which contains the element.
	 * @param element
	 *     The model element.
	 * 
	 * @see IElementStatusConstants
	 */
	public ElementStatus(int code, String message, EObject root, EObject element) {
		super(ERROR, VoclToOclConverterPlugin.PLUGIN_ID, code, message, null);
		this.root = root;
		this.element = element;
	}
	
	/**
	 * Returns the root element of the EMF model which contains the element. 
	 */
	public EObject getRoot() {
		return root;
	}
	
	/**
	 * Returns the element.
	 */
	public EObject getElement() {
		return element;
	}
}
